@@ -1,3 +1,7 @@
+/**
+ * $Id: GraphEditor.java,v 1.5 2010-08-10 11:43:36 david Exp $
+ * Copyright (c) 2006-2010, Gaudenz Alder, David Benson
+ */
 package com.mxgraph.examples.swing;
 
 import java.awt.Color;
@@ -166,12 +170,20 @@ public class GraphEditor extends BasicGraphEditor
 								GraphEditor.class
 										.getResource("/com/mxgraph/examples/swing/images/cylinder.png")),
 						"shape=cylinder", 120, 160, "");
-		shapesPalette.addTemplate("Actor", new ImageIcon(GraphEditor.class
-				.getResource("/com/mxgraph/examples/swing/images/actor.png")),
-				"shape=actor", 120, 160, "");
-		shapesPalette.addTemplate("Cloud", new ImageIcon(GraphEditor.class
-				.getResource("/com/mxgraph/examples/swing/images/cloud.png")),
-				"ellipse;shape=cloud", 160, 120, "");
+		shapesPalette
+				.addTemplate(
+						"Actor",
+						new ImageIcon(
+								GraphEditor.class
+										.getResource("/com/mxgraph/examples/swing/images/actor.png")),
+						"shape=actor", 120, 160, "");
+		shapesPalette
+				.addTemplate(
+						"Cloud",
+						new ImageIcon(
+								GraphEditor.class
+										.getResource("/com/mxgraph/examples/swing/images/cloud.png")),
+						"ellipse;shape=cloud", 160, 120, "");
 
 		shapesPalette
 				.addEdgeTemplate(
@@ -201,18 +213,30 @@ public class GraphEditor extends BasicGraphEditor
 								GraphEditor.class
 										.getResource("/com/mxgraph/examples/swing/images/entity.png")),
 						"entity", 100, 100, "");
-		shapesPalette.addEdgeTemplate("Arrow", new ImageIcon(GraphEditor.class
-				.getResource("/com/mxgraph/examples/swing/images/arrow.png")),
-				"arrow", 120, 120, "");
+		shapesPalette
+				.addEdgeTemplate(
+						"Arrow",
+						new ImageIcon(
+								GraphEditor.class
+										.getResource("/com/mxgraph/examples/swing/images/arrow.png")),
+						"arrow", 120, 120, "");
 
-		imagesPalette.addTemplate("Bell", new ImageIcon(GraphEditor.class
-				.getResource("/com/mxgraph/examples/swing/images/bell.png")),
-				"image;image=/com/mxgraph/examples/swing/images/bell.png", 50,
-				50, "Bell");
-		imagesPalette.addTemplate("Box", new ImageIcon(GraphEditor.class
-				.getResource("/com/mxgraph/examples/swing/images/box.png")),
-				"image;image=/com/mxgraph/examples/swing/images/box.png", 50,
-				50, "Box");
+		imagesPalette
+				.addTemplate(
+						"Bell",
+						new ImageIcon(
+								GraphEditor.class
+										.getResource("/com/mxgraph/examples/swing/images/bell.png")),
+						"image;image=/com/mxgraph/examples/swing/images/bell.png",
+						50, 50, "Bell");
+		imagesPalette
+				.addTemplate(
+						"Box",
+						new ImageIcon(
+								GraphEditor.class
+										.getResource("/com/mxgraph/examples/swing/images/box.png")),
+						"image;image=/com/mxgraph/examples/swing/images/box.png",
+						50, 50, "Box");
 		imagesPalette
 				.addTemplate(
 						"Cube",
@@ -237,14 +261,22 @@ public class GraphEditor extends BasicGraphEditor
 										.getResource("/com/mxgraph/examples/swing/images/earth.png")),
 						"roundImage;image=/com/mxgraph/examples/swing/images/earth.png",
 						50, 50, "Earth");
-		imagesPalette.addTemplate("Gear", new ImageIcon(GraphEditor.class
-				.getResource("/com/mxgraph/examples/swing/images/gear.png")),
-				"roundImage;image=/com/mxgraph/examples/swing/images/gear.png",
-				50, 50, "Gear");
-		imagesPalette.addTemplate("Home", new ImageIcon(GraphEditor.class
-				.getResource("/com/mxgraph/examples/swing/images/house.png")),
-				"image;image=/com/mxgraph/examples/swing/images/house.png", 50,
-				50, "Home");
+		imagesPalette
+				.addTemplate(
+						"Gear",
+						new ImageIcon(
+								GraphEditor.class
+										.getResource("/com/mxgraph/examples/swing/images/gear.png")),
+						"roundImage;image=/com/mxgraph/examples/swing/images/gear.png",
+						50, 50, "Gear");
+		imagesPalette
+				.addTemplate(
+						"Home",
+						new ImageIcon(
+								GraphEditor.class
+										.getResource("/com/mxgraph/examples/swing/images/house.png")),
+						"image;image=/com/mxgraph/examples/swing/images/house.png",
+						50, 50, "Home");
 		imagesPalette
 				.addTemplate(
 						"Package",
@@ -261,10 +293,14 @@ public class GraphEditor extends BasicGraphEditor
 										.getResource("/com/mxgraph/examples/swing/images/printer.png")),
 						"image;image=/com/mxgraph/examples/swing/images/printer.png",
 						50, 50, "Printer");
-		imagesPalette.addTemplate("Server", new ImageIcon(GraphEditor.class
-				.getResource("/com/mxgraph/examples/swing/images/server.png")),
-				"image;image=/com/mxgraph/examples/swing/images/server.png",
-				50, 50, "Server");
+		imagesPalette
+				.addTemplate(
+						"Server",
+						new ImageIcon(
+								GraphEditor.class
+										.getResource("/com/mxgraph/examples/swing/images/server.png")),
+						"image;image=/com/mxgraph/examples/swing/images/server.png",
+						50, 50, "Server");
 		imagesPalette
 				.addTemplate(
 						"Workplace",
@@ -322,10 +358,14 @@ public class GraphEditor extends BasicGraphEditor
 										.getResource("/com/mxgraph/examples/swing/images/inclusive.png")),
 						"rhombusImage;image=/com/mxgraph/examples/swing/images/inclusive.png",
 						80, 80, "Inclusive");
-		symbolsPalette.addTemplate("Link", new ImageIcon(GraphEditor.class
-				.getResource("/com/mxgraph/examples/swing/images/link.png")),
-				"roundImage;image=/com/mxgraph/examples/swing/images/link.png",
-				80, 80, "Link");
+		symbolsPalette
+				.addTemplate(
+						"Link",
+						new ImageIcon(
+								GraphEditor.class
+										.getResource("/com/mxgraph/examples/swing/images/link.png")),
+						"roundImage;image=/com/mxgraph/examples/swing/images/link.png",
+						80, 80, "Link");
 		symbolsPalette
 				.addTemplate(
 						"Merge",
@@ -350,10 +390,14 @@ public class GraphEditor extends BasicGraphEditor
 										.getResource("/com/mxgraph/examples/swing/images/multiple.png")),
 						"roundImage;image=/com/mxgraph/examples/swing/images/multiple.png",
 						80, 80, "Multiple");
-		symbolsPalette.addTemplate("Rule", new ImageIcon(GraphEditor.class
-				.getResource("/com/mxgraph/examples/swing/images/rule.png")),
-				"roundImage;image=/com/mxgraph/examples/swing/images/rule.png",
-				80, 80, "Rule");
+		symbolsPalette
+				.addTemplate(
+						"Rule",
+						new ImageIcon(
+								GraphEditor.class
+										.getResource("/com/mxgraph/examples/swing/images/rule.png")),
+						"roundImage;image=/com/mxgraph/examples/swing/images/rule.png",
+						80, 80, "Rule");
 		symbolsPalette
 				.addTemplate(
 						"Terminate",
@@ -373,8 +417,8 @@ public class GraphEditor extends BasicGraphEditor
 	}
 
 	/**
-	 * 
-	 */
+	* 
+	*/
 	public static class CustomGraphComponent extends mxGraphComponent
 	{
 
