@@ -35,8 +35,8 @@ import javax.swing.UIManager;
 import com.mxgraph.layout.mxCircleLayout;
 import com.mxgraph.layout.mxCompactTreeLayout;
 import com.mxgraph.layout.mxEdgeLabelLayout;
-import com.mxgraph.layout.mxFastOrganicLayout;
 import com.mxgraph.layout.mxIGraphLayout;
+import com.mxgraph.layout.mxOrganicLayout;
 import com.mxgraph.layout.mxParallelEdgeLayout;
 import com.mxgraph.layout.mxPartitionLayout;
 import com.mxgraph.layout.mxStackLayout;
@@ -45,6 +45,7 @@ import com.mxgraph.swing.mxGraphComponent;
 import com.mxgraph.swing.mxGraphOutline;
 import com.mxgraph.swing.handler.mxKeyboardHandler;
 import com.mxgraph.swing.handler.mxRubberband;
+import com.mxgraph.swing.util.mxMorphing;
 import com.mxgraph.util.mxEvent;
 import com.mxgraph.util.mxEventObject;
 import com.mxgraph.util.mxRectangle;
@@ -68,7 +69,14 @@ public class BasicGraphEditor extends JPanel
 	 */
 	static
 	{
-		mxResources.add("com/mxgraph/examples/swing/resources/editor");
+		try
+		{
+			mxResources.add("com/mxgraph/examples/swing/resources/editor");
+		}
+		catch (Exception e)
+		{
+			// ignore
+		}
 	}
 
 	/**
@@ -107,7 +115,7 @@ public class BasicGraphEditor extends JPanel
 	protected File currentFile;
 
 	/**
-	 * 
+	 * Flag indicating whether the current graph has been modified 
 	 */
 	protected boolean modified = false;
 
@@ -155,7 +163,10 @@ public class BasicGraphEditor extends JPanel
 		// Stores a reference to the graph and creates the command history
 		graphComponent = component;
 		final mxGraph graph = graphComponent.getGraph();
-		undoManager = new mxUndoManager();
+		undoManager = createUndoManager();
+
+		// Do not change the scale and translation after files have been loaded
+		graph.setResetViewOnRootChange(false);
 
 		// Updates the modified flag if the graph model changes
 		graph.getModel().addListener(mxEvent.CHANGE, changeTracker);
@@ -222,6 +233,14 @@ public class BasicGraphEditor extends JPanel
 		updateTitle();
 	}
 
+	/**
+	 * 
+	 */
+	protected mxUndoManager createUndoManager()
+	{
+		return new mxUndoManager();
+	}
+
 	/**
 	 * 
 	 */
@@ -337,8 +356,8 @@ public class BasicGraphEditor extends JPanel
 	{
 		Point pt = SwingUtilities.convertPoint(e.getComponent(), e.getPoint(),
 				graphComponent);
-		JCheckBoxMenuItem item = new JCheckBoxMenuItem(mxResources
-				.get("magnifyPage"));
+		JCheckBoxMenuItem item = new JCheckBoxMenuItem(
+				mxResources.get("magnifyPage"));
 		item.setSelected(graphOutline.isFitPage());
 
 		item.addActionListener(new ActionListener()
@@ -353,8 +372,8 @@ public class BasicGraphEditor extends JPanel
 			}
 		});
 
-		JCheckBoxMenuItem item2 = new JCheckBoxMenuItem(mxResources
-				.get("showLabels"));
+		JCheckBoxMenuItem item2 = new JCheckBoxMenuItem(
+				mxResources.get("showLabels"));
 		item2.setSelected(graphOutline.isDrawLabels());
 
 		item2.addActionListener(new ActionListener()
@@ -369,8 +388,8 @@ public class BasicGraphEditor extends JPanel
 			}
 		});
 
-		JCheckBoxMenuItem item3 = new JCheckBoxMenuItem(mxResources
-				.get("buffering"));
+		JCheckBoxMenuItem item3 = new JCheckBoxMenuItem(
+				mxResources.get("buffering"));
 		item3.setSelected(graphOutline.isTripleBuffered());
 
 		item3.addActionListener(new ActionListener()
@@ -380,8 +399,7 @@ public class BasicGraphEditor extends JPanel
 			 */
 			public void actionPerformed(ActionEvent e)
 			{
-				graphOutline
-						.setTripleBuffered(!graphOutline.isTripleBuffered());
+				graphOutline.setTripleBuffered(!graphOutline.isTripleBuffered());
 				graphOutline.repaint();
 			}
 		});
@@ -563,7 +581,7 @@ public class BasicGraphEditor extends JPanel
 
 	/**
 	 * 
-	 * @return
+	 * @return whether or not the current graph has been modified
 	 */
 	public boolean isModified()
 	{
@@ -585,6 +603,14 @@ public class BasicGraphEditor extends JPanel
 	{
 		return graphOutline;
 	}
+	
+	/**
+	 * 
+	 */
+	public JTabbedPane getLibraryPane()
+	{
+		return libraryPane;
+	}
 
 	/**
 	 * 
@@ -598,7 +624,7 @@ public class BasicGraphEditor extends JPanel
 	 * 
 	 * @param name
 	 * @param action
-	 * @return
+	 * @return a new Action bound to the specified string name
 	 */
 	public Action bind(String name, final Action action)
 	{
@@ -609,7 +635,7 @@ public class BasicGraphEditor extends JPanel
 	 * 
 	 * @param name
 	 * @param action
-	 * @return
+	 * @return a new Action bound to the specified string name and icon
 	 */
 	@SuppressWarnings("serial")
 	public Action bind(String name, final Action action, String iconUrl)
@@ -732,16 +758,16 @@ public class BasicGraphEditor extends JPanel
 	}
 
 	/**
-	 * Creates and executes the specified layout.
+	 * Creates an action that executes the specified layout.
 	 * 
 	 * @param key Key to be used for getting the label from mxResources and also
 	 * to create the layout instance for the commercial graph editor example.
-	 * @return
+	 * @return an action that executes the specified layout
 	 */
 	@SuppressWarnings("serial")
-	public Action graphLayout(final String key)
+	public Action graphLayout(final String key, boolean animate)
 	{
-		final mxIGraphLayout layout = createLayout(key);
+		final mxIGraphLayout layout = createLayout(key, animate);
 
 		if (layout != null)
 		{
@@ -749,23 +775,41 @@ public class BasicGraphEditor extends JPanel
 			{
 				public void actionPerformed(ActionEvent e)
 				{
-					if (layout != null)
+					final mxGraph graph = graphComponent.getGraph();
+					Object cell = graph.getSelectionCell();
+
+					if (cell == null
+							|| graph.getModel().getChildCount(cell) == 0)
 					{
-						Object cell = graphComponent.getGraph()
-								.getSelectionCell();
-
-						if (cell == null
-								|| graphComponent.getGraph().getModel()
-										.getChildCount(cell) == 0)
-						{
-							cell = graphComponent.getGraph().getDefaultParent();
-						}
+						cell = graph.getDefaultParent();
+					}
 
+					graph.getModel().beginUpdate();
+					try
+					{
 						long t0 = System.currentTimeMillis();
 						layout.execute(cell);
 						status("Layout: " + (System.currentTimeMillis() - t0)
 								+ " ms");
 					}
+					finally
+					{
+						mxMorphing morph = new mxMorphing(graphComponent, 20,
+								1.2, 20);
+
+						morph.addListener(mxEvent.DONE, new mxIEventListener()
+						{
+
+							public void invoke(Object sender, mxEventObject evt)
+							{
+								graph.getModel().endUpdate();
+							}
+
+						});
+
+						morph.startAnimation();
+					}
+
 				}
 
 			};
@@ -777,8 +821,8 @@ public class BasicGraphEditor extends JPanel
 
 				public void actionPerformed(ActionEvent e)
 				{
-					JOptionPane.showMessageDialog(graphComponent, mxResources
-							.get("noLayout"));
+					JOptionPane.showMessageDialog(graphComponent,
+							mxResources.get("noLayout"));
 				}
 
 			};
@@ -788,7 +832,7 @@ public class BasicGraphEditor extends JPanel
 	/**
 	 * Creates a layout instance for the given identifier.
 	 */
-	protected mxIGraphLayout createLayout(String ident)
+	protected mxIGraphLayout createLayout(String ident, boolean animate)
 	{
 		mxIGraphLayout layout = null;
 
@@ -822,7 +866,7 @@ public class BasicGraphEditor extends JPanel
 			}
 			else if (ident.equals("organicLayout"))
 			{
-				layout = new mxFastOrganicLayout(graph);
+				layout = new mxOrganicLayout(graph);
 			}
 			if (ident.equals("verticalPartition"))
 			{
