@@ -1,6 +1,6 @@
 /*
- * $Id: EditorActions.java,v 1.6 2009/12/08 19:52:50 gaudenz Exp $
- * Copyright (c) 2001-2009, JGraph Ltd
+ * $Id: EditorActions.java,v 1.29 2010-09-09 07:02:56 david Exp $
+ * Copyright (c) 2001-2010, Gaudenz Alder, David Benson
  * 
  * All rights reserved.
  * 
@@ -11,6 +11,7 @@ package com.mxgraph.examples.swing.editor;
 
 import java.awt.Color;
 import java.awt.Component;
+import java.awt.Image;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.image.BufferedImage;
@@ -21,15 +22,22 @@ import java.awt.print.PrinterJob;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.FilenameFilter;
 import java.io.IOException;
 import java.lang.reflect.Method;
+import java.net.URLDecoder;
+import java.net.URLEncoder;
 import java.util.HashSet;
+import java.util.Map;
 
 import javax.imageio.ImageIO;
 import javax.swing.AbstractAction;
 import javax.swing.ImageIcon;
 import javax.swing.JCheckBoxMenuItem;
 import javax.swing.JColorChooser;
+import javax.swing.JComponent;
 import javax.swing.JEditorPane;
 import javax.swing.JFileChooser;
 import javax.swing.JOptionPane;
@@ -41,12 +49,20 @@ import javax.swing.text.html.HTMLDocument;
 import javax.swing.text.html.HTMLEditorKit;
 
 import org.w3c.dom.Document;
+import org.w3c.dom.Node;
 
 import com.mxgraph.analysis.mxDistanceCostFunction;
 import com.mxgraph.analysis.mxGraphAnalysis;
+import com.mxgraph.canvas.mxGraphics2DCanvas;
 import com.mxgraph.io.mxCodec;
+import com.mxgraph.io.mxGdCodec;
+import com.mxgraph.io.mxVdxCodec;
+import com.mxgraph.io.gd.mxGdDocument;
 import com.mxgraph.model.mxCell;
+import com.mxgraph.model.mxGraphModel;
 import com.mxgraph.model.mxIGraphModel;
+import com.mxgraph.shape.mxIShape;
+import com.mxgraph.shape.mxStencilShape;
 import com.mxgraph.swing.mxGraphComponent;
 import com.mxgraph.swing.mxGraphOutline;
 import com.mxgraph.swing.handler.mxConnectionHandler;
@@ -56,15 +72,16 @@ import com.mxgraph.util.mxCellRenderer;
 import com.mxgraph.util.mxConstants;
 import com.mxgraph.util.mxResources;
 import com.mxgraph.util.mxUtils;
+import com.mxgraph.util.png.mxPNGzTXtDecoder;
+import com.mxgraph.util.png.mxPngEncodeParam;
+import com.mxgraph.util.png.mxPngImageEncoder;
 import com.mxgraph.view.mxGraph;
 
 /**
- * @author Administrator
- * 
+ *
  */
 public class EditorActions
 {
-
 	/**
 	 * 
 	 * @param e
@@ -129,7 +146,6 @@ public class EditorActions
 				}
 			});
 		}
-
 	}
 
 	/**
@@ -229,7 +245,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class ExitAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -242,7 +257,6 @@ public class EditorActions
 				editor.exit();
 			}
 		}
-
 	}
 
 	/**
@@ -251,7 +265,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class StylesheetAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -281,13 +294,12 @@ public class EditorActions
 
 				if (doc != null)
 				{
-					codec.decode(doc.getDocumentElement(), graph
-							.getStylesheet());
+					codec.decode(doc.getDocumentElement(),
+							graph.getStylesheet());
 					graph.refresh();
 				}
 			}
 		}
-
 	}
 
 	/**
@@ -322,7 +334,6 @@ public class EditorActions
 				graphComponent.setZoomPolicy(zoomPolicy);
 			}
 		}
-
 	}
 
 	/**
@@ -357,7 +368,6 @@ public class EditorActions
 				graphComponent.repaint();
 			}
 		}
-
 	}
 
 	/**
@@ -366,7 +376,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class GridColorAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -377,8 +386,8 @@ public class EditorActions
 				mxGraphComponent graphComponent = (mxGraphComponent) e
 						.getSource();
 				Color newColor = JColorChooser.showDialog(graphComponent,
-						mxResources.get("gridColor"), graphComponent
-								.getGridColor());
+						mxResources.get("gridColor"),
+						graphComponent.getGridColor());
 
 				if (newColor != null)
 				{
@@ -387,7 +396,6 @@ public class EditorActions
 				}
 			}
 		}
-
 	}
 
 	/**
@@ -396,7 +404,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class ScaleAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -440,7 +447,6 @@ public class EditorActions
 				}
 			}
 		}
-
 	}
 
 	/**
@@ -449,7 +455,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class PageSetupAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -470,7 +475,6 @@ public class EditorActions
 				}
 			}
 		}
-
 	}
 
 	/**
@@ -479,7 +483,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class PrintAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -512,7 +515,6 @@ public class EditorActions
 				}
 			}
 		}
-
 	}
 
 	/**
@@ -521,7 +523,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class SaveAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -540,6 +541,55 @@ public class EditorActions
 			this.showDialog = showDialog;
 		}
 
+		/**
+		 * Saves XML+PNG format.
+		 */
+		protected void saveXmlPng(BasicGraphEditor editor, String filename,
+				Color bg) throws IOException
+		{
+			mxGraphComponent graphComponent = editor.getGraphComponent();
+			mxGraph graph = graphComponent.getGraph();
+
+			// Creates the image for the PNG file
+			BufferedImage image = mxCellRenderer.createBufferedImage(graph,
+					null, 1, bg, graphComponent.isAntiAlias(), null,
+					graphComponent.getCanvas());
+
+			// Creates the URL-encoded XML data
+			mxCodec codec = new mxCodec();
+			String xml = URLEncoder.encode(
+					mxUtils.getXml(codec.encode(graph.getModel())), "UTF-8");
+			mxPngEncodeParam param = mxPngEncodeParam
+					.getDefaultEncodeParam(image);
+			param.setCompressedText(new String[] { "mxGraphModel", xml });
+
+			// Saves as a PNG file
+			FileOutputStream outputStream = new FileOutputStream(new File(
+					filename));
+			try
+			{
+				mxPngImageEncoder encoder = new mxPngImageEncoder(outputStream,
+						param);
+
+				if (image != null)
+				{
+					encoder.encode(image);
+
+					editor.setModified(false);
+					editor.setCurrentFile(new File(filename));
+				}
+				else
+				{
+					JOptionPane.showMessageDialog(graphComponent,
+							mxResources.get("noImageData"));
+				}
+			}
+			finally
+			{
+				outputStream.close();
+			}
+		}
+
 		/**
 		 * 
 		 */
@@ -552,9 +602,12 @@ public class EditorActions
 				mxGraphComponent graphComponent = editor.getGraphComponent();
 				mxGraph graph = graphComponent.getGraph();
 				FileFilter selectedFilter = null;
+				DefaultFileFilter xmlPngFilter = new DefaultFileFilter(".png",
+						"PNG+XML " + mxResources.get("file") + " (.png)");
 				FileFilter vmlFileFilter = new DefaultFileFilter(".html",
 						"VML " + mxResources.get("file") + " (.html)");
 				String filename = null;
+				boolean dialogShown = false;
 
 				if (showDialog || editor.getCurrentFile() == null)
 				{
@@ -576,12 +629,16 @@ public class EditorActions
 					JFileChooser fc = new JFileChooser(wd);
 
 					// Adds the default file format
-					FileFilter defaultFilter = new DefaultFileFilter(".mxe",
-							"mxGraph Editor " + mxResources.get("file")
-									+ " (.mxe)");
+					FileFilter defaultFilter = xmlPngFilter;
 					fc.addChoosableFileFilter(defaultFilter);
 
 					// Adds special vector graphics formats and HTML
+					fc.addChoosableFileFilter(new DefaultFileFilter(".mxe",
+							"mxGraph Editor " + mxResources.get("file")
+									+ " (.mxe)"));
+					fc.addChoosableFileFilter(new DefaultFileFilter(".txt",
+							"Graph Drawing " + mxResources.get("file")
+									+ " (.txt)"));
 					fc.addChoosableFileFilter(new DefaultFileFilter(".svg",
 							"SVG " + mxResources.get("file") + " (.svg)"));
 					fc.addChoosableFileFilter(vmlFileFilter);
@@ -611,11 +668,11 @@ public class EditorActions
 					}
 
 					// Adds filter that accepts all supported image formats
-					fc
-							.addChoosableFileFilter(new DefaultFileFilter.ImageFileFilter(
-									mxResources.get("allImages")));
+					fc.addChoosableFileFilter(new DefaultFileFilter.ImageFileFilter(
+							mxResources.get("allImages")));
 					fc.setFileFilter(defaultFilter);
 					int rc = fc.showDialog(null, mxResources.get("save"));
+					dialogShown = true;
 
 					if (rc != JFileChooser.APPROVE_OPTION)
 					{
@@ -687,6 +744,13 @@ public class EditorActions
 						editor.setModified(false);
 						editor.setCurrentFile(new File(filename));
 					}
+					else if (ext.equalsIgnoreCase("txt"))
+					{
+						String content = mxGdCodec.encode(graph)
+								.getDocumentString();
+
+						mxUtils.writeFile(content, filename);
+					}
 					else
 					{
 						Color bg = null;
@@ -700,19 +764,28 @@ public class EditorActions
 							bg = graphComponent.getBackground();
 						}
 
-						BufferedImage image = mxCellRenderer
-								.createBufferedImage(graph, null, 1, bg,
-										graphComponent.isAntiAlias(), null,
-										graphComponent.getCanvas());
-
-						if (image != null)
+						if (selectedFilter == xmlPngFilter
+								|| (editor.getCurrentFile() != null
+										&& ext.equalsIgnoreCase("png") && !dialogShown))
 						{
-							ImageIO.write(image, ext, new File(filename));
+							saveXmlPng(editor, filename, bg);
 						}
 						else
 						{
-							JOptionPane.showMessageDialog(graphComponent,
-									mxResources.get("noImageData"));
+							BufferedImage image = mxCellRenderer
+									.createBufferedImage(graph, null, 1, bg,
+											graphComponent.isAntiAlias(), null,
+											graphComponent.getCanvas());
+
+							if (image != null)
+							{
+								ImageIO.write(image, ext, new File(filename));
+							}
+							else
+							{
+								JOptionPane.showMessageDialog(graphComponent,
+										mxResources.get("noImageData"));
+							}
 						}
 					}
 				}
@@ -720,8 +793,7 @@ public class EditorActions
 				{
 					ex.printStackTrace();
 					JOptionPane.showMessageDialog(graphComponent,
-							ex.toString(),
-							mxResources.get("error"),
+							ex.toString(), mxResources.get("error"),
 							JOptionPane.ERROR_MESSAGE);
 				}
 			}
@@ -734,7 +806,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class SelectShortestPathAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -796,12 +867,11 @@ public class EditorActions
 				}
 				else
 				{
-					JOptionPane.showMessageDialog(graphComponent, mxResources
-							.get("noSourceAndTargetSelected"));
+					JOptionPane.showMessageDialog(graphComponent,
+							mxResources.get("noSourceAndTargetSelected"));
 				}
 			}
 		}
-
 	}
 
 	/**
@@ -810,7 +880,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class SelectSpanningTreeAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -855,7 +924,6 @@ public class EditorActions
 				graph.setSelectionCells(mst);
 			}
 		}
-
 	}
 
 	/**
@@ -864,7 +932,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class ToggleDirtyAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -880,36 +947,12 @@ public class EditorActions
 
 	}
 
-	/**
-	 *
-	 */
-	@SuppressWarnings("serial")
-	public static class ToggleImagePreviewAction extends AbstractAction
-	{
-
-		/**
-		 * 
-		 */
-		public void actionPerformed(ActionEvent e)
-		{
-			if (e.getSource() instanceof mxGraphComponent)
-			{
-				mxGraphComponent graphComponent = (mxGraphComponent) e
-						.getSource();
-				graphComponent.getGraphHandler().setImagePreview(
-						!graphComponent.getGraphHandler().isImagePreview());
-			}
-		}
-
-	}
-
 	/**
 	 *
 	 */
 	@SuppressWarnings("serial")
 	public static class ToggleConnectModeAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -924,7 +967,6 @@ public class EditorActions
 				handler.setHandleEnabled(!handler.isHandleEnabled());
 			}
 		}
-
 	}
 
 	/**
@@ -1041,7 +1083,6 @@ public class EditorActions
 				graphComponent.repaint();
 			}
 		}
-
 	}
 
 	/**
@@ -1050,7 +1091,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class TogglePropertyItem extends JCheckBoxMenuItem
 	{
-
 		/**
 		 * 
 		 */
@@ -1109,7 +1149,6 @@ public class EditorActions
 						update(target, fieldname);
 					}
 				}
-
 			};
 
 			if (target instanceof mxGraphComponent)
@@ -1131,19 +1170,27 @@ public class EditorActions
 		 */
 		public void update(Object target, String fieldname)
 		{
-			try
+			if (target != null && fieldname != null)
 			{
-				Method getter = target.getClass().getMethod("is" + fieldname);
-				Object current = getter.invoke(target);
-
-				if (current instanceof Boolean)
+				try
 				{
-					setSelected(((Boolean) current).booleanValue());
+					Method getter = target.getClass().getMethod(
+							"is" + fieldname);
+
+					if (getter != null)
+					{
+						Object current = getter.invoke(target);
+
+						if (current instanceof Boolean)
+						{
+							setSelected(((Boolean) current).booleanValue());
+						}
+					}
+				}
+				catch (Exception e)
+				{
+					// ignore
 				}
-			}
-			catch (Exception e)
-			{
-				e.printStackTrace();
 			}
 		}
 
@@ -1152,43 +1199,46 @@ public class EditorActions
 		 */
 		public void execute(Object target, String fieldname, boolean refresh)
 		{
-			try
+			if (target != null && fieldname != null)
 			{
-				Method getter = target.getClass().getMethod("is" + fieldname);
-				Method setter = target.getClass().getMethod("set" + fieldname,
-						new Class[] { boolean.class });
-
-				Object current = getter.invoke(target);
-
-				if (current instanceof Boolean)
+				try
 				{
-					boolean value = !((Boolean) current).booleanValue();
-					setter.invoke(target, value);
-					setSelected(value);
-				}
+					Method getter = target.getClass().getMethod(
+							"is" + fieldname);
+					Method setter = target.getClass().getMethod(
+							"set" + fieldname, new Class[] { boolean.class });
 
-				if (refresh)
-				{
-					mxGraph graph = null;
+					Object current = getter.invoke(target);
 
-					if (target instanceof mxGraph)
+					if (current instanceof Boolean)
 					{
-						graph = (mxGraph) target;
-					}
-					else if (target instanceof mxGraphComponent)
-					{
-						graph = ((mxGraphComponent) target).getGraph();
+						boolean value = !((Boolean) current).booleanValue();
+						setter.invoke(target, value);
+						setSelected(value);
 					}
 
-					graph.refresh();
+					if (refresh)
+					{
+						mxGraph graph = null;
+
+						if (target instanceof mxGraph)
+						{
+							graph = (mxGraph) target;
+						}
+						else if (target instanceof mxGraphComponent)
+						{
+							graph = ((mxGraphComponent) target).getGraph();
+						}
+
+						graph.refresh();
+					}
+				}
+				catch (Exception e)
+				{
+					// ignore
 				}
-			}
-			catch (Exception e)
-			{
-				e.printStackTrace();
 			}
 		}
-
 	}
 
 	/**
@@ -1197,7 +1247,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class HistoryAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1287,9 +1336,8 @@ public class EditorActions
 								.getDocument();
 						document.remove(start, (ende - start));
 						editorKit.insertHTML(document, start, ((bold) ? "<b>"
-								: "<i>")
-								+ text + ((bold) ? "</b>" : "</i>"), 0, 0,
-								(bold) ? HTML.Tag.B : HTML.Tag.I);
+								: "<i>") + text + ((bold) ? "</b>" : "</i>"),
+								0, 0, (bold) ? HTML.Tag.B : HTML.Tag.I);
 					}
 					catch (Exception ex)
 					{
@@ -1326,7 +1374,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class WarningAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1350,8 +1397,8 @@ public class EditorActions
 				}
 				else
 				{
-					JOptionPane.showMessageDialog(graphComponent, mxResources
-							.get("noCellSelected"));
+					JOptionPane.showMessageDialog(graphComponent,
+							mxResources.get("noCellSelected"));
 				}
 			}
 		}
@@ -1363,7 +1410,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class NewAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1374,10 +1420,11 @@ public class EditorActions
 			if (editor != null)
 			{
 				if (!editor.isModified()
-						|| JOptionPane.showConfirmDialog(editor, mxResources
-								.get("loseChanges")) == JOptionPane.YES_OPTION)
+						|| JOptionPane.showConfirmDialog(editor,
+								mxResources.get("loseChanges")) == JOptionPane.YES_OPTION)
 				{
 					mxGraph graph = editor.getGraphComponent().getGraph();
+
 					// Check modified flag and display save dialog
 					mxCell root = new mxCell();
 					root.insert(new mxCell());
@@ -1385,6 +1432,132 @@ public class EditorActions
 
 					editor.setModified(false);
 					editor.setCurrentFile(null);
+					editor.getGraphComponent().zoomAndCenter();
+				}
+			}
+		}
+	}
+
+	/**
+	 *
+	 */
+	@SuppressWarnings("serial")
+	public static class ImportAction extends AbstractAction
+	{
+		/**
+		 * 
+		 */
+		protected String lastDir;
+
+		/**
+		 * Loads and registers the shape as a new shape in mxGraphics2DCanvas and
+		 * adds a new entry to use that shape in the specified palette
+		 * @param palette The palette to add the shape to.
+		 * @param nodeXml The raw XML of the shape
+		 * @param path The path to the directory the shape exists in
+		 * @return the string name of the shape
+		 */
+		public String addStencilShape(EditorPalette palette, String nodeXml,
+				String path)
+		{
+
+			// Some editors place a 3 byte BOM at the start of files
+			// Ensure the first char is a "<"
+			int lessthanIndex = nodeXml.indexOf("<");
+			nodeXml = nodeXml.substring(lessthanIndex);
+			mxStencilShape newShape = new mxStencilShape(nodeXml);
+			String name = newShape.getName();
+			ImageIcon icon = null;
+
+			if (path != null)
+			{
+				String iconPath = path + newShape.getIconPath();
+				icon = new ImageIcon(iconPath);
+			}
+
+			// Registers the shape in the canvas shape registry
+			mxGraphics2DCanvas.putShape(name, newShape);
+
+			if (palette != null && icon != null)
+			{
+				palette.addTemplate(name, icon, "shape=" + name, 80, 80, "");
+			}
+			
+			return name;
+		}
+
+		/**
+		 * 
+		 */
+		public void actionPerformed(ActionEvent e)
+		{
+			BasicGraphEditor editor = getEditor(e);
+
+			if (editor != null)
+			{
+				String wd = (lastDir != null) ? lastDir : System
+						.getProperty("user.dir");
+
+				JFileChooser fc = new JFileChooser(wd);
+
+				fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
+
+				// Adds file filter for Dia shape import
+				fc.addChoosableFileFilter(new DefaultFileFilter(".shape",
+						"Dia Shape " + mxResources.get("file") + " (.shape)"));
+
+				int rc = fc.showDialog(null, mxResources.get("importStencil"));
+
+				if (rc == JFileChooser.APPROVE_OPTION)
+				{
+					lastDir = fc.getSelectedFile().getParent();
+
+					try
+					{
+						if (fc.getSelectedFile().isDirectory())
+						{
+							EditorPalette palette = editor.insertPalette(fc
+									.getSelectedFile().getName());
+
+							for (File f : fc.getSelectedFile().listFiles(
+									new FilenameFilter()
+									{
+										public boolean accept(File dir,
+												String name)
+										{
+											return name.toLowerCase().endsWith(
+													".shape");
+										}
+									}))
+							{
+								String nodeXml = mxUtils.readFile(f.getAbsolutePath());
+								addStencilShape(palette, nodeXml, f.getParent() + File.separator);
+							}
+
+							JComponent scrollPane = (JComponent) palette
+									.getParent().getParent();
+							editor.getLibraryPane().setSelectedComponent(
+									scrollPane);
+
+							// FIXME: Need to update the size of the palette to force a layout
+							// update. Re/in/validate of palette or parent does not work.
+							//editor.getLibraryPane().revalidate();
+						}
+						else
+						{
+							String nodeXml = mxUtils.readFile(fc
+									.getSelectedFile().getAbsolutePath());
+							String name = addStencilShape(null, nodeXml, null);
+
+							JOptionPane.showMessageDialog(editor, mxResources
+									.get("stencilImported",
+											new String[] { name }));
+						}
+					}
+					catch (IOException e1)
+					{
+						e1.printStackTrace();
+					}
 				}
 			}
 		}
@@ -1401,6 +1574,99 @@ public class EditorActions
 		 */
 		protected String lastDir;
 
+		/**
+		 * 
+		 */
+		protected void resetEditor(BasicGraphEditor editor)
+		{
+			editor.setModified(false);
+			editor.getUndoManager().clear();
+			editor.getGraphComponent().zoomAndCenter();
+		}
+
+		/**
+		 * Reads XML+PNG format.
+		 */
+		protected void openXmlPng(BasicGraphEditor editor, File file)
+				throws IOException
+		{
+			Map<String, String> text = mxPNGzTXtDecoder
+					.decodezTXt(new FileInputStream(file));
+
+			if (text != null)
+			{
+				String value = text.get("mxGraphModel");
+
+				if (value != null)
+				{
+					Document document = mxUtils.parseXml(URLDecoder.decode(
+							value, "UTF-8"));
+					mxCodec codec = new mxCodec(document);
+					codec.decode(document.getDocumentElement(), editor
+							.getGraphComponent().getGraph().getModel());
+					editor.setCurrentFile(file);
+					resetEditor(editor);
+
+					return;
+				}
+			}
+
+			JOptionPane.showMessageDialog(editor,
+					mxResources.get("imageContainsNoDiagramData"));
+		}
+
+		/**
+		 * @throws IOException 
+		 * 
+		 */
+		protected void openVdx(BasicGraphEditor editor, File file,
+				Document document)
+		{
+			mxGraph graph = editor.getGraphComponent().getGraph();
+
+			// Replaces file extension with .mxe
+			String filename = file.getName();
+			filename = filename.substring(0, filename.length() - 4) + ".mxe";
+
+			if (new File(filename).exists()
+					&& JOptionPane.showConfirmDialog(editor,
+							mxResources.get("overwriteExistingFile")) != JOptionPane.YES_OPTION)
+			{
+				return;
+			}
+
+			((mxGraphModel) graph.getModel()).clear();
+			mxVdxCodec.decode(document, graph);
+			editor.getGraphComponent().zoomAndCenter();
+			editor.setCurrentFile(new File(lastDir + "/" + filename));
+		}
+
+		/**
+		 * @throws IOException
+		 *
+		 */
+		protected void openGD(BasicGraphEditor editor, File file,
+				mxGdDocument document)
+		{
+			mxGraph graph = editor.getGraphComponent().getGraph();
+
+			// Replaces file extension with .mxe
+			String filename = file.getName();
+			filename = filename.substring(0, filename.length() - 4) + ".mxe";
+
+			if (new File(filename).exists()
+					&& JOptionPane.showConfirmDialog(editor,
+							mxResources.get("overwriteExistingFile")) != JOptionPane.YES_OPTION)
+			{
+				return;
+			}
+
+			((mxGraphModel) graph.getModel()).clear();
+			mxGdCodec.decode(document, graph);
+			editor.getGraphComponent().zoomAndCenter();
+			editor.setCurrentFile(new File(lastDir + "/" + filename));
+		}
+
 		/**
 		 * 
 		 */
@@ -1411,10 +1677,10 @@ public class EditorActions
 			if (editor != null)
 			{
 				if (!editor.isModified()
-						|| JOptionPane.showConfirmDialog(editor, mxResources
-								.get("loseChanges")) == JOptionPane.YES_OPTION)
+						|| JOptionPane.showConfirmDialog(editor,
+								mxResources.get("loseChanges")) == JOptionPane.YES_OPTION)
 				{
-					mxGraph graph = mxGraphActions.getGraph(e);
+					mxGraph graph = editor.getGraphComponent().getGraph();
 
 					if (graph != null)
 					{
@@ -1424,13 +1690,43 @@ public class EditorActions
 						JFileChooser fc = new JFileChooser(wd);
 
 						// Adds file filter for supported file format
-						FileFilter defaultFilter = new DefaultFileFilter(
-								".mxe", "mxGraph Editor "
-										+ mxResources.get("file") + " (.mxe)");
+						DefaultFileFilter defaultFilter = new DefaultFileFilter(
+								".mxe", mxResources.get("allSupportedFormats")
+										+ " (.mxe, .png, .vdx)")
+						{
+
+							public boolean accept(File file)
+							{
+								String lcase = file.getName().toLowerCase();
+
+								return super.accept(file)
+										|| lcase.endsWith(".png")
+										|| lcase.endsWith(".vdx");
+							}
+						};
 						fc.addChoosableFileFilter(defaultFilter);
 
-						int rc = fc.showDialog(null, mxResources
-								.get("openFile"));
+						fc.addChoosableFileFilter(new DefaultFileFilter(".mxe",
+								"mxGraph Editor " + mxResources.get("file")
+										+ " (.mxe)"));
+						fc.addChoosableFileFilter(new DefaultFileFilter(".png",
+								"PNG+XML  " + mxResources.get("file")
+										+ " (.png)"));
+
+						// Adds file filter for VDX import
+						fc.addChoosableFileFilter(new DefaultFileFilter(".vdx",
+								"XML Drawing  " + mxResources.get("file")
+										+ " (.vdx)"));
+
+						// Adds file filter for GD import
+						fc.addChoosableFileFilter(new DefaultFileFilter(".txt",
+								"Graph Drawing  " + mxResources.get("file")
+										+ " (.txt)"));
+
+						fc.setFileFilter(defaultFilter);
+
+						int rc = fc.showDialog(null,
+								mxResources.get("openFile"));
 
 						if (rc == JFileChooser.APPROVE_OPTION)
 						{
@@ -1438,22 +1734,56 @@ public class EditorActions
 
 							try
 							{
-								Document document = mxUtils.parse(mxUtils
-										.readFile(fc.getSelectedFile()
-												.getAbsolutePath()));
+								if (fc.getSelectedFile().getAbsolutePath()
+										.toLowerCase().endsWith(".png"))
+								{
+									openXmlPng(editor, fc.getSelectedFile());
+								}
+								else if (fc.getSelectedFile().getAbsolutePath()
+										.toLowerCase().endsWith(".txt"))
+								{
+									mxGdDocument document = new mxGdDocument();
+									document.parse(mxUtils.readFile(fc
+											.getSelectedFile()
+											.getAbsolutePath()));
+									openGD(editor, fc.getSelectedFile(),
+											document);
+								}
+								else
+								{
+									Document document = mxUtils
+											.parseXml(mxUtils.readFile(fc
+													.getSelectedFile()
+													.getAbsolutePath()));
 
-								mxCodec codec = new mxCodec(document);
-								codec.decode(document.getDocumentElement(),
-										graph.getModel());
+									if (fc.getSelectedFile().getAbsolutePath()
+											.toLowerCase().endsWith(".vdx"))
+									{
+										openVdx(editor, fc.getSelectedFile(),
+												document);
+									}
+									else
+									{
+										mxCodec codec = new mxCodec(document);
+										codec.decode(
+												document.getDocumentElement(),
+												graph.getModel());
+										editor.setCurrentFile(fc
+												.getSelectedFile());
+									}
 
-								editor.setModified(false);
-								editor.setCurrentFile(fc.getSelectedFile());
+									resetEditor(editor);
+								}
 							}
-							catch (IOException e1)
+							catch (IOException ex)
 							{
-								e1.printStackTrace();
+								ex.printStackTrace();
+								JOptionPane.showMessageDialog(
+										editor.getGraphComponent(),
+										ex.toString(),
+										mxResources.get("error"),
+										JOptionPane.ERROR_MESSAGE);
 							}
-
 						}
 					}
 				}
@@ -1467,7 +1797,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class ToggleAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1517,7 +1846,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class SetLabelPositionAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1577,7 +1905,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class SetStyleAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1612,7 +1939,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class KeyValueAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1657,7 +1983,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class PromptValueAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1709,7 +2034,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class AlignCellsAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1744,7 +2068,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class AutosizeAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1754,7 +2077,21 @@ public class EditorActions
 
 			if (graph != null && !graph.isSelectionEmpty())
 			{
-				graph.updateCellSize(graph.getSelectionCell());
+				Object[] cells = graph.getSelectionCells();
+				mxIGraphModel model = graph.getModel();
+
+				model.beginUpdate();
+				try
+				{
+					for (int i = 0; i < cells.length; i++)
+					{
+						graph.updateCellSize(cells[i]);
+					}
+				}
+				finally
+				{
+					model.endUpdate();
+				}
 			}
 		}
 	}
@@ -1765,7 +2102,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class ColorAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1812,7 +2148,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class BackgroundImageAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1835,8 +2170,14 @@ public class EditorActions
 					}
 					else
 					{
-						graphComponent.setBackgroundImage(new ImageIcon(mxUtils
-								.loadImage(value)));
+						Image background = mxUtils.loadImage(value);
+						// Incorrect URLs will result in no image.
+						// TODO provide feedback that the URL is not correct
+						if (background != null)
+						{
+							graphComponent.setBackgroundImage(new ImageIcon(
+									background));
+						}
 					}
 
 					// Forces a repaint of the outline
@@ -1852,7 +2193,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class BackgroundAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1883,7 +2223,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class PageBackgroundAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1913,7 +2252,6 @@ public class EditorActions
 	@SuppressWarnings("serial")
 	public static class StyleAction extends AbstractAction
 	{
-
 		/**
 		 * 
 		 */
@@ -1927,9 +2265,9 @@ public class EditorActions
 				String initial = graph.getModel().getStyle(
 						graph.getSelectionCell());
 				String value = (String) JOptionPane.showInputDialog(
-						graphComponent, mxResources.get("style"), mxResources
-								.get("style"), JOptionPane.PLAIN_MESSAGE, null,
-						null, initial);
+						graphComponent, mxResources.get("style"),
+						mxResources.get("style"), JOptionPane.PLAIN_MESSAGE,
+						null, null, initial);
 
 				if (value != null)
 				{
@@ -1938,5 +2276,4 @@ public class EditorActions
 			}
 		}
 	}
-
 }
