@@ -1,5 +1,5 @@
 /**
- * $Id: mxSvgCanvas.java,v 1.49 2010/01/13 10:43:46 gaudenz Exp $
+ * $Id: mxSvgCanvas.java,v 1.56 2010-08-02 13:14:43 david Exp $
  * Copyright (c) 2007, Gaudenz Alder
  */
 package com.mxgraph.canvas;
@@ -13,7 +13,9 @@ import org.w3c.dom.Element;
 
 import com.mxgraph.util.mxConstants;
 import com.mxgraph.util.mxPoint;
+import com.mxgraph.util.mxRectangle;
 import com.mxgraph.util.mxUtils;
+import com.mxgraph.view.mxCellState;
 
 /**
  * An implementation of a canvas that uses SVG for painting. This canvas
@@ -76,41 +78,69 @@ public class mxSvgCanvas extends mxBasicCanvas
 
 	/*
 	 * (non-Javadoc)
-	 * @see com.mxgraph.canvas.mxICanvas#drawVertex(int, int, int, int, java.util.Hashtable)
+	 * @see com.mxgraph.canvas.mxICanvas#drawCell()
 	 */
-	public Object drawVertex(int x, int y, int w, int h,
-			Map<String, Object> style)
+	public Object drawCell(mxCellState state)
 	{
+		Map<String, Object> style = state.getStyle();
 		Element elem = null;
 
-		x += translate.x;
-		y += translate.y;
-
-		if (!mxUtils.getString(style, mxConstants.STYLE_SHAPE, "").equals(
-				mxConstants.SHAPE_SWIMLANE))
+		if (state.getAbsolutePointCount() > 1)
 		{
-			elem = drawShape(x, y, w, h, style);
+			List<mxPoint> pts = state.getAbsolutePoints();
+
+			// Transpose all points by cloning into a new array
+			pts = mxUtils.translatePoints(pts, translate.x, translate.y);
+
+			// Draws the line
+			elem = drawLine(pts, style);
+
+			// Applies opacity
+			float opacity = mxUtils.getFloat(style, mxConstants.STYLE_OPACITY,
+					100);
+
+			if (opacity != 100)
+			{
+				String value = String.valueOf(opacity / 100);
+				elem.setAttribute("fill-opacity", value);
+				elem.setAttribute("stroke-opacity", value);
+			}
 		}
 		else
 		{
-			int start = (int) Math.round(mxUtils.getInt(style,
-					mxConstants.STYLE_STARTSIZE, mxConstants.DEFAULT_STARTSIZE)
-					* scale);
+			int x = (int) state.getX() + translate.x;
+			int y = (int) state.getY() + translate.y;
+			int w = (int) state.getWidth();
+			int h = (int) state.getHeight();
 
-			// Removes some styles to draw the content area
-			Map<String, Object> cloned = new Hashtable<String, Object>(style);
-			cloned.remove(mxConstants.STYLE_FILLCOLOR);
-			cloned.remove(mxConstants.STYLE_ROUNDED);
-
-			if (mxUtils.isTrue(style, mxConstants.STYLE_HORIZONTAL, true))
+			if (!mxUtils.getString(style, mxConstants.STYLE_SHAPE, "").equals(
+					mxConstants.SHAPE_SWIMLANE))
 			{
-				elem = drawShape(x, y, w, start, style);
-				drawShape(x, y + start, w, h - start, cloned);
+				elem = drawShape(x, y, w, h, style);
 			}
 			else
 			{
-				elem = drawShape(x, y, start, h, style);
-				drawShape(x + start, y, w - start, h, cloned);
+				int start = (int) Math.round(mxUtils.getInt(style,
+						mxConstants.STYLE_STARTSIZE,
+						mxConstants.DEFAULT_STARTSIZE)
+						* scale);
+
+				// Removes some styles to draw the content area
+				Map<String, Object> cloned = new Hashtable<String, Object>(
+						style);
+				cloned.remove(mxConstants.STYLE_FILLCOLOR);
+				cloned.remove(mxConstants.STYLE_ROUNDED);
+
+				if (mxUtils.isTrue(style, mxConstants.STYLE_HORIZONTAL, true))
+				{
+					elem = drawShape(x, y, w, start, style);
+					drawShape(x, y + start, w, h - start, cloned);
+				}
+				else
+				{
+					elem = drawShape(x, y, start, h, style);
+					drawShape(x + start, y, w - start, h, cloned);
+				}
 			}
 		}
 
@@ -119,40 +149,19 @@ public class mxSvgCanvas extends mxBasicCanvas
 
 	/*
 	 * (non-Javadoc)
-	 * @see com.mxgraph.canvas.mxICanvas#drawEdge(java.util.List, java.util.Hashtable)
+	 * @see com.mxgraph.canvas.mxICanvas#drawLabel()
 	 */
-	public Object drawEdge(List<mxPoint> pts, Map<String, Object> style)
+	public Object drawLabel(String label, mxCellState state, boolean html)
 	{
-		// Transpose all points by cloning into a new array
-		pts = mxUtils.translatePoints(pts, translate.x, translate.y);
+		mxRectangle bounds = state.getLabelBounds();
 
-		// Draws the line
-		Element elem = drawLine(pts, style);
-
-		// Applies opacity
-		float opacity = mxUtils.getFloat(style, mxConstants.STYLE_OPACITY, 100);
-
-		if (opacity != 100)
-		{
-			String value = String.valueOf(opacity / 100);
-			elem.setAttribute("fill-opacity", value);
-			elem.setAttribute("stroke-opacity", value);
-		}
-
-		return elem;
-	}
-
-	/*
-	 * (non-Javadoc)
-	 * @see com.mxgraph.canvas.mxICanvas#drawLabel(java.lang.String, int, int, int, int, java.util.Hashtable, boolean)
-	 */
-	public Object drawLabel(String label, int x, int y, int w, int h,
-			Map<String, Object> style, boolean isHtml)
-	{
 		if (drawLabels)
 		{
-			x += translate.x;
-			y += translate.y;
+			int x = (int) bounds.getX() + translate.x;
+			int y = (int) bounds.getY() + translate.y;
+			int w = (int) bounds.getWidth();
+			int h = (int) bounds.getHeight();
+			Map<String, Object> style = state.getStyle();
 
 			return drawText(label, x, y, w, h, style);
 		}
@@ -197,7 +206,7 @@ public class mxSvgCanvas extends mxBasicCanvas
 				elem.setAttribute("width", String.valueOf(w));
 				elem.setAttribute("height", String.valueOf(h));
 
-				elem.setAttributeNS(mxConstants.NS_XLINK, "href", img);
+				elem.setAttributeNS(mxConstants.NS_XLINK, "xlink:href", img);
 			}
 		}
 		else if (shape.equals(mxConstants.SHAPE_LINE))
@@ -209,12 +218,12 @@ public class mxSvgCanvas extends mxBasicCanvas
 			if (direction.equals(mxConstants.DIRECTION_EAST)
 					|| direction.equals(mxConstants.DIRECTION_WEST))
 			{
-				int mid = (int) (y + h / 2);
+				int mid = (y + h / 2);
 				d = "M " + x + " " + mid + " L " + (x + w) + " " + mid;
 			}
 			else
 			{
-				int mid = (int) (x + w / 2);
+				int mid = (x + w / 2);
 				d = "M " + mid + " " + y + " L " + mid + " " + (y + h);
 			}
 
@@ -473,6 +482,11 @@ public class mxSvgCanvas extends mxBasicCanvas
 			}
 		}
 
+		if (mxUtils.isTrue(style, mxConstants.STYLE_DASHED))
+		{
+			elem.setAttribute("stroke-dasharray", "3, 3");
+		}
+
 		appendSvgElement(elem);
 
 		return elem;
@@ -492,7 +506,7 @@ public class mxSvgCanvas extends mxBasicCanvas
 
 		String strokeColor = mxUtils.getString(style,
 				mxConstants.STYLE_STROKECOLOR);
-		float tmpStroke = (float) (mxUtils.getFloat(style,
+		float tmpStroke = (mxUtils.getFloat(style,
 				mxConstants.STYLE_STROKEWIDTH, 1));
 		float strokeWidth = (float) (tmpStroke * scale);
 
@@ -507,7 +521,7 @@ public class mxSvgCanvas extends mxBasicCanvas
 
 			if (marker != null)
 			{
-				float size = (float) (mxUtils.getFloat(style,
+				float size = (mxUtils.getFloat(style,
 						mxConstants.STYLE_STARTSIZE,
 						mxConstants.DEFAULT_MARKERSIZE));
 				offset = drawMarker(group, marker, pt, p0, size, tmpStroke,
@@ -543,7 +557,7 @@ public class mxSvgCanvas extends mxBasicCanvas
 
 			if (marker != null)
 			{
-				float size = (float) (mxUtils.getFloat(style,
+				float size = (mxUtils.getFloat(style,
 						mxConstants.STYLE_ENDSIZE,
 						mxConstants.DEFAULT_MARKERSIZE));
 				offset = drawMarker(group, marker, pt, pe, size, tmpStroke,
@@ -588,6 +602,11 @@ public class mxSvgCanvas extends mxBasicCanvas
 			path.setAttribute("fill", "none");
 			path.setAttribute("stroke-width", String.valueOf(strokeWidth));
 
+			if (mxUtils.isTrue(style, mxConstants.STYLE_DASHED))
+			{
+				path.setAttribute("stroke-dasharray", "3, 3");
+			}
+
 			group.appendChild(path);
 			appendSvgElement(group);
 		}
