@@ -1,5 +1,5 @@
 /**
- * $Id: mxObjectCodec.java,v 1.26 2009/12/05 14:03:03 gaudenz Exp $
+ * $Id: mxObjectCodec.java,v 1.34 2010-08-31 11:23:12 gaudenz Exp $
  * Copyright (c) 2006, Gaudenz Alder
  */
 package com.mxgraph.io;
@@ -133,6 +133,29 @@ public class mxObjectCodec
 		}
 	}
 
+	/**
+	 * Returns the name used for the nodenames and lookup of the codec when
+	 * classes are encoded and nodes are decoded. For classes to work with
+	 * this the codec registry automatically adds an alias for the classname
+	 * if that is different than what this returns. The default implementation
+	 * returns the classname of the template class.
+	 * 
+	 * Here is an example on how to use this for renaming mxCell nodes:
+	 * <code>
+	 * mxCodecRegistry.register(new mxCellCodec()
+	 * {
+	 *   public String getName()
+	 *   {
+	 *     return "anotherName";
+	 *   }
+	 * });
+	 * </code>
+	 */
+	public String getName()
+	{
+		return mxCodecRegistry.getName(getTemplate());
+	}
+
 	/**
 	 * Returns the template object associated with this codec.
 	 * 
@@ -156,7 +179,14 @@ public class mxObjectCodec
 
 		try
 		{
-			obj = template.getClass().newInstance();
+			if (template.getClass().isEnum())
+			{
+				obj = template.getClass().getEnumConstants()[0];
+			}
+			else
+			{
+				obj = template.getClass().newInstance();
+			}
 
 			// Special case: Check if the collection
 			// should be a map. This is if the first
@@ -172,7 +202,7 @@ public class mxObjectCodec
 				if (node != null && node instanceof Element
 						&& ((Element) node).hasAttribute("as"))
 				{
-					obj = new Hashtable();
+					obj = new Hashtable<Object, Object>();
 				}
 			}
 		}
@@ -271,8 +301,7 @@ public class mxObjectCodec
 	 */
 	public Node encode(mxCodec enc, Object obj)
 	{
-		String name = mxCodecRegistry.getName(obj);
-		Node node = enc.document.createElement(name);
+		Node node = enc.document.createElement(getName());
 
 		obj = beforeEncode(enc, obj, node);
 		encodeObject(enc, obj, node);
@@ -305,7 +334,7 @@ public class mxObjectCodec
 	 */
 	protected void encodeFields(mxCodec enc, Object obj, Node node)
 	{
-		Class type = obj.getClass();
+		Class<?> type = obj.getClass();
 
 		while (type != null)
 		{
@@ -358,7 +387,7 @@ public class mxObjectCodec
 		}
 		else if (obj instanceof Collection)
 		{
-			Iterator it = ((Collection) obj).iterator();
+			Iterator<?> it = ((Collection<?>) obj).iterator();
 
 			while (it.hasNext())
 			{
@@ -392,8 +421,7 @@ public class mxObjectCodec
 				if (tmp == null)
 				{
 					System.err.println("mxObjectCodec.encode: No ID for "
-							+ mxCodecRegistry.getName(obj) + "."
-							+ fieldname + "=" + value);
+							+ getName() + "." + fieldname + "=" + value);
 					return; // exit
 				}
 
@@ -488,9 +516,8 @@ public class mxObjectCodec
 		}
 		else
 		{
-			System.err.println("mxObjectCodec.encode: No node for "
-					+ mxCodecRegistry.getName(obj) + "." + attr
-					+ ": " + value);
+			System.err.println("mxObjectCodec.encode: No node for " + getName()
+					+ "." + attr + ": " + value);
 		}
 	}
 
@@ -510,48 +537,48 @@ public class mxObjectCodec
 	/**
 	 * Converts XML attribute values to object of the given type.
 	 */
-	protected Object convertValueFromXml(Class type, Object value)
+	protected Object convertValueFromXml(Class<?> type, Object value)
 	{
-		if (value instanceof String && type.isPrimitive())
+		if (value instanceof String)
 		{
 			String tmp = (String) value;
 
-			if (type.equals(boolean.class))
+			if (type.equals(boolean.class) || type == Boolean.class)
 			{
 				if (tmp.equals("1") || tmp.equals("0"))
 				{
 					tmp = (tmp.equals("1")) ? "true" : "false";
 				}
 
-				value = new Boolean(tmp);
+				value = Boolean.valueOf(tmp);
 			}
-			else if (type.equals(char.class))
+			else if (type.equals(char.class) || type == Character.class)
 			{
-				value = new Character(tmp.charAt(0));
+				value = Character.valueOf(tmp.charAt(0));
 			}
-			else if (type.equals(byte.class))
+			else if (type.equals(byte.class) || type == Byte.class)
 			{
-				value = new Byte(tmp);
+				value = Byte.valueOf(tmp);
 			}
-			else if (type.equals(short.class))
+			else if (type.equals(short.class) || type == Short.class)
 			{
-				value = new Short(tmp);
+				value = Short.valueOf(tmp);
 			}
-			else if (type.equals(int.class))
+			else if (type.equals(int.class) || type == Integer.class)
 			{
-				value = new Integer(tmp);
+				value = Integer.valueOf(tmp);
 			}
-			else if (type.equals(long.class))
+			else if (type.equals(long.class) || type == Long.class)
 			{
-				value = new Long(tmp);
+				value = Long.valueOf(tmp);
 			}
-			else if (type.equals(float.class))
+			else if (type.equals(float.class) || type == Float.class)
 			{
-				value = new Float(tmp);
+				value = Float.valueOf(tmp);
 			}
-			else if (type.equals(double.class))
+			else if (type.equals(double.class) || type == Double.class)
 			{
-				value = new Double(tmp);
+				value = Double.valueOf(tmp);
 			}
 		}
 
@@ -605,7 +632,7 @@ public class mxObjectCodec
 	 */
 	protected Field getField(Object obj, String fieldname)
 	{
-		Class type = obj.getClass();
+		Class<?> type = obj.getClass();
 
 		while (type != null)
 		{
@@ -674,7 +701,7 @@ public class mxObjectCodec
 	 */
 	protected Method getMethod(Object obj, String methodname, Class[] params)
 	{
-		Class type = obj.getClass();
+		Class<?> type = obj.getClass();
 
 		while (type != null)
 		{
@@ -767,13 +794,13 @@ public class mxObjectCodec
 				try
 				{
 					Method method = getAccessor(obj, field, false);
-					Class type = method.getParameterTypes()[0];
+					Class<?> type = method.getParameterTypes()[0];
 					value = convertValueFromXml(type, value);
 
 					// Converts collection to a typed array before setting
 					if (type.isArray() && value instanceof Collection)
 					{
-						Collection coll = (Collection) value;
+						Collection<?> coll = (Collection<?>) value;
 						value = coll.toArray((Object[]) Array.newInstance(type
 								.getComponentType(), coll.size()));
 					}
@@ -954,8 +981,7 @@ public class mxObjectCodec
 				if (tmp == null)
 				{
 					System.err.println("mxObjectCodec.decode: No object for "
-							+ mxCodecRegistry.getName(obj) + "."
-							+ fieldname + "=" + value);
+							+ getName() + "." + fieldname + "=" + value);
 					return; // exit
 				}
 
@@ -1019,7 +1045,7 @@ public class mxObjectCodec
 				// Collections are cleared
 				else if (template instanceof Collection)
 				{
-					((Collection) template).clear();
+					((Collection<?>) template).clear();
 				}
 
 				value = dec.decode(child, template);
