@@ -1,12 +1,6 @@
 /*
- * Copyright (c) 2005-2006, David Benson
- *
- * All rights reserved.
- *
- * This file is licensed under the JGraph software license, a copy of which
- * will have been provided to you in the file LICENSE at the root of your
- * installation directory. If you are unable to locate this file please
- * contact JGraph sales for another copy.
+ * $Id: mxGraphHierarchyModel.java,v 1.10 2010-08-02 13:26:46 david Exp $
+ * Copyright (c) 2005-2010, David Benson, Gaudenz Alder
  */
 package com.mxgraph.layout.hierarchical.model;
 
@@ -162,7 +156,7 @@ public class mxGraphHierarchyModel
 						Object realEdge = iter2.next();
 						Object targetCell = graph.getView().getVisibleTerminal(
 								realEdge, false);
-						mxGraphHierarchyNode internalTargetCell = (mxGraphHierarchyNode) vertexMapper
+						mxGraphHierarchyNode internalTargetCell = vertexMapper
 								.get(targetCell);
 
 						if (internalTargetCell != null
@@ -238,7 +232,7 @@ public class mxGraphHierarchyModel
 					Object realEdge = iter2.next();
 					Object targetCell = graph.getView().getVisibleTerminal(
 							realEdge, false);
-					mxGraphHierarchyNode internalTargetCell = (mxGraphHierarchyNode) vertexMapper
+					mxGraphHierarchyNode internalTargetCell = vertexMapper
 							.get(targetCell);
 
 					if (internalTargetCell != null
@@ -540,7 +534,7 @@ public class mxGraphHierarchyModel
 
 					while (iter3.hasNext())
 					{
-						mxGraphHierarchyEdge internalEdge = (mxGraphHierarchyEdge) iter3
+						mxGraphHierarchyEdge internalEdge = iter3
 								.next();
 						// Assign unique stamp ( y/m/d/h )
 						internalEdge.temp[0] = 5270620;
@@ -597,7 +591,7 @@ public class mxGraphHierarchyModel
 			// Tighten the rank 0 nodes as far as possible
 			for (int i = 0; i < startNodesCopy.size(); i++)
 			{
-				mxGraphHierarchyNode internalNode = (mxGraphHierarchyNode) startNodesCopy
+				mxGraphHierarchyNode internalNode = startNodesCopy
 						.get(i);
 				int currentMinLayer = 1000000;
 				Iterator<mxGraphHierarchyEdge> iter2 = internalNode.connectsAsTarget
@@ -605,7 +599,7 @@ public class mxGraphHierarchyModel
 
 				while (iter2.hasNext())
 				{
-					mxGraphHierarchyEdge internalEdge = (mxGraphHierarchyEdge) iter2
+					mxGraphHierarchyEdge internalEdge = iter2
 							.next();
 					mxGraphHierarchyNode otherNode = internalEdge.source;
 					internalNode.temp[0] = Math.min(currentMinLayer,
@@ -621,7 +615,7 @@ public class mxGraphHierarchyModel
 			iter = internalNodes.iterator();
 			while (iter.hasNext())
 			{
-				mxGraphHierarchyNode internalNode = (mxGraphHierarchyNode) iter
+				mxGraphHierarchyNode internalNode = iter
 						.next();
 				// Mark the node as not having had a layer assigned
 				internalNode.temp[0] -= maxRank;
@@ -675,7 +669,7 @@ public class mxGraphHierarchyModel
 				if (seen == 0 && node.maxRank < 0 && node.minRank < 0)
 				{
 					rankList[node.temp[0]]
-							.add((mxGraphAbstractHierarchyCell) cell);
+							.add(cell);
 					node.maxRank = node.temp[0];
 					node.minRank = node.temp[0];
 
@@ -685,15 +679,15 @@ public class mxGraphHierarchyModel
 
 				if (parent != null && connectingEdge != null)
 				{
-					int parentToCellRankDifference = ((mxGraphHierarchyNode) parent).maxRank
+					int parentToCellRankDifference = (parent).maxRank
 							- node.maxRank;
 
 					if (parentToCellRankDifference > 1)
 					{
 						// There are ranks in between the parent and current cell
-						mxGraphHierarchyEdge edge = (mxGraphHierarchyEdge) connectingEdge;
-						edge.maxRank = ((mxGraphHierarchyNode) parent).maxRank;
-						edge.minRank = ((mxGraphHierarchyNode) cell).maxRank;
+						mxGraphHierarchyEdge edge = connectingEdge;
+						edge.maxRank = (parent).maxRank;
+						edge.minRank = (cell).maxRank;
 						edge.temp = new int[parentToCellRankDifference - 1];
 						edge.x = new double[parentToCellRankDifference - 1];
 						edge.y = new double[parentToCellRankDifference - 1];
@@ -920,12 +914,13 @@ public class mxGraphHierarchyModel
 		 *            the parent cell the current cell
 		 * @param cell
 		 *            the current cell visited
-		 * @param previousSibling
-		 *            the last neighbour cell found
+		 * @param connectingEdge
+		 *            the edge that led the last cell visited to this cell
 		 * @param layer
 		 *            the current layer of the tree
-		 * @param sibling
-		 *            the number of sibling to the current cell found
+		 * @param seen
+		 *            an int indicating whether this cell has been seen
+		 *            previously
 		 */
 		public void visit(mxGraphHierarchyNode parent, mxGraphHierarchyNode cell,
 				mxGraphHierarchyEdge connectingEdge, int layer, int seen);
