@@ -204,7 +204,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 
 						while (iter.hasNext())
 						{
-							mxGraphAbstractHierarchyCell cell = (mxGraphAbstractHierarchyCell) iter
+							mxGraphAbstractHierarchyCell cell = iter
 									.next();
 							cell.setX(j, cell.getGeneralPurposeVariable(j));
 						}
@@ -224,7 +224,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 
 						while (iter.hasNext())
 						{
-							mxGraphAbstractHierarchyCell cell = (mxGraphAbstractHierarchyCell) iter
+							mxGraphAbstractHierarchyCell cell = iter
 									.next();
 							cell.setGeneralPurposeVariable(j, (int) cell
 									.getX(j));
@@ -256,7 +256,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 
 		for (int i = 0; i <= model.maxRank; i++)
 		{
-			mxGraphHierarchyRank rankSet = (mxGraphHierarchyRank) model.ranks
+			mxGraphHierarchyRank rankSet = model.ranks
 					.get(new Integer(i));
 			rank[i] = rankSet.toArray();
 
@@ -283,7 +283,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 
 		while (!nodeList.isEmpty() && count <= maxTries)
 		{
-			WeightedCellSorter cellWrapper = (WeightedCellSorter) nodeList
+			WeightedCellSorter cellWrapper = nodeList
 					.getFirst();
 			mxGraphAbstractHierarchyCell cell = cellWrapper.cell;
 
@@ -387,7 +387,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 				for (int i = 0; i < nextLayerConnectedCells.length; i++)
 				{
 					mxGraphAbstractHierarchyCell connectedCell = (mxGraphAbstractHierarchyCell) nextLayerConnectedCells[i];
-					WeightedCellSorter connectedCellWrapper = (WeightedCellSorter) map
+					WeightedCellSorter connectedCellWrapper = map
 							.get(connectedCell);
 
 					if (connectedCellWrapper != null)
@@ -404,7 +404,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 				for (int i = 0; i < previousLayerConnectedCells.length; i++)
 				{
 					mxGraphAbstractHierarchyCell connectedCell = (mxGraphAbstractHierarchyCell) previousLayerConnectedCells[i];
-					WeightedCellSorter connectedCellWrapper = (WeightedCellSorter) map
+					WeightedCellSorter connectedCellWrapper = map
 							.get(connectedCell);
 
 					if (connectedCellWrapper != null)
@@ -467,7 +467,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 	protected void rankMedianPosition(int rankValue,
 			mxGraphHierarchyModel model, int nextRankValue)
 	{
-		mxGraphHierarchyRank rankSet = (mxGraphHierarchyRank) model.ranks
+		mxGraphHierarchyRank rankSet = model.ranks
 				.get(new Integer(rankValue));
 		Object[] rank = rankSet.toArray();
 		// Form an array of the order in which the cell are to be processed
@@ -548,7 +548,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 
 			for (int j = weightedValues[i].rankIndex - 1; j >= 0;)
 			{
-				WeightedCellSorter weightedValue = (WeightedCellSorter) cellMap
+				WeightedCellSorter weightedValue = cellMap
 						.get(rank[j]);
 
 				if (weightedValue != null)
@@ -581,7 +581,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 
 			for (int j = weightedValues[i].rankIndex + 1; j < weightedValues.length;)
 			{
-				WeightedCellSorter weightedValue = (WeightedCellSorter) cellMap
+				WeightedCellSorter weightedValue = cellMap
 						.get(rank[j]);
 
 				if (weightedValue != null)
@@ -613,7 +613,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 			{
 				cell
 						.setGeneralPurposeVariable(rankValue,
-								(int) medianNextLevel);
+								medianNextLevel);
 			}
 			else if (medianNextLevel < leftLimit)
 			{
@@ -774,7 +774,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 
 		while (iter.hasNext())
 		{
-			mxGraphAbstractHierarchyCell cell = (mxGraphAbstractHierarchyCell) iter
+			mxGraphAbstractHierarchyCell cell = iter
 					.next();
 			if (cell.isVertex())
 			{
@@ -870,7 +870,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 
 			while (iter.hasNext())
 			{
-				mxGraphAbstractHierarchyCell cell = (mxGraphAbstractHierarchyCell) iter
+				mxGraphAbstractHierarchyCell cell = iter
 						.next();
 
 				if (cell.isVertex())
@@ -958,7 +958,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 
 			while (iter.hasNext())
 			{
-				mxGraphAbstractHierarchyCell cell = (mxGraphAbstractHierarchyCell) iter
+				mxGraphAbstractHierarchyCell cell = iter
 						.next();
 				cell.setY(rankValue, y);
 			}
@@ -983,7 +983,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 			Iterator<mxGraphAbstractHierarchyCell> iter = rank.iterator();
 			while (iter.hasNext())
 			{
-				mxGraphAbstractHierarchyCell cell = (mxGraphAbstractHierarchyCell) iter
+				mxGraphAbstractHierarchyCell cell = iter
 						.next();
 
 				if (cell.isVertex())
@@ -1201,7 +1201,6 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 	 * Utility class that stores a collection of vertices and edge points within
 	 * a certain area. This area includes the buffer lengths of cells.
 	 */
-	@SuppressWarnings("serial")
 	protected class AreaSpatialCache extends Rectangle2D.Double
 	{
 		public Set<Object> cells = new HashSet<Object>();
