@@ -106,7 +106,7 @@ public class mxMedianHybridCrossingReduction implements
 
 					for (int k = 0; k < rank.size(); k++)
 					{
-						mxGraphAbstractHierarchyCell cell = (mxGraphAbstractHierarchyCell) iter
+						mxGraphAbstractHierarchyCell cell = iter
 								.next();
 						nestedBestRanks[j][cell.getGeneralPurposeVariable(j)] = cell;
 					}
@@ -127,7 +127,7 @@ public class mxMedianHybridCrossingReduction implements
 
 					for (int k = 0; k < rank.size(); k++)
 					{
-						mxGraphAbstractHierarchyCell cell = (mxGraphAbstractHierarchyCell) iter
+						mxGraphAbstractHierarchyCell cell = iter
 								.next();
 						cell.setGeneralPurposeVariable(j, k);
 					}
@@ -302,7 +302,7 @@ public class mxMedianHybridCrossingReduction implements
 
 				for (int j = 0; j < orderedCells.length; j++)
 				{
-					mxGraphAbstractHierarchyCell cell = (mxGraphAbstractHierarchyCell) iter
+					mxGraphAbstractHierarchyCell cell = iter
 							.next();
 					orderedCells[cell.getGeneralPurposeVariable(i)] = cell;
 				}
@@ -343,13 +343,13 @@ public class mxMedianHybridCrossingReduction implements
 
 						for (int k = 0; k < leftAbovePositions.length; k++)
 						{
-							leftAbovePositions[k] = ((mxGraphAbstractHierarchyCell) leftCellAboveConnections
-									.get(k)).getGeneralPurposeVariable(i + 1);
+							leftAbovePositions[k] = leftCellAboveConnections
+									.get(k).getGeneralPurposeVariable(i + 1);
 						}
 
 						for (int k = 0; k < leftBelowPositions.length; k++)
 						{
-							leftBelowPositions[k] = ((mxGraphAbstractHierarchyCell) leftCellBelowConnections
+							leftBelowPositions[k] = (leftCellBelowConnections
 									.get(k)).getGeneralPurposeVariable(i - 1);
 						}
 					}
@@ -375,13 +375,13 @@ public class mxMedianHybridCrossingReduction implements
 
 					for (int k = 0; k < rightAbovePositions.length; k++)
 					{
-						rightAbovePositions[k] = ((mxGraphAbstractHierarchyCell) rightCellAboveConnections
+						rightAbovePositions[k] = (rightCellAboveConnections
 								.get(k)).getGeneralPurposeVariable(i + 1);
 					}
 
 					for (int k = 0; k < rightBelowPositions.length; k++)
 					{
-						rightBelowPositions[k] = ((mxGraphAbstractHierarchyCell) rightCellBelowConnections
+						rightBelowPositions[k] = (rightCellBelowConnections
 								.get(k)).getGeneralPurposeVariable(i - 1);
 					}
 
@@ -496,7 +496,7 @@ public class mxMedianHybridCrossingReduction implements
 
 		for (int i = 0; i < numCellsForRank; i++)
 		{
-			mxGraphAbstractHierarchyCell cell = (mxGraphAbstractHierarchyCell) nestedBestRanks[rankValue][i];
+			mxGraphAbstractHierarchyCell cell = nestedBestRanks[rankValue][i];
 			medianValues[i] = new MedianCellSorter();
 			medianValues[i].cell = cell;
 
@@ -574,7 +574,7 @@ public class mxMedianHybridCrossingReduction implements
 
 		while (iter.hasNext())
 		{
-			medianValues[arrayCount++] = ((mxGraphAbstractHierarchyCell) iter
+			medianValues[arrayCount++] = (iter
 					.next()).getGeneralPurposeVariable(rankValue);
 		}
 
