@@ -1,5 +1,5 @@
 /**
- * $Id: mxOrganicLayout.java,v 1.5 2009/11/24 12:00:28 gaudenz Exp $
+ * $Id: mxOrganicLayout.java,v 1.6 2010-08-02 13:14:02 david Exp $
  * Copyright (c) 2007-2009, JGraph Ltd
  */
 
@@ -470,11 +470,11 @@ public class mxOrganicLayout extends mxGraphLayout
 			// Check if either end of the edge is not connected
 			if (sourceCell != null)
 			{
-				source = (Integer) vertexMap.get(sourceCell);
+				source = vertexMap.get(sourceCell);
 			}
 			if (targetCell != null)
 			{
-				target = (Integer) vertexMap.get(targetCell);
+				target = vertexMap.get(targetCell);
 			}
 			if (source != null)
 			{
@@ -510,8 +510,8 @@ public class mxOrganicLayout extends mxGraphLayout
 
 		for (int i = 0; i < triesPerCell; i++)
 		{
-			double angle = (double) i
-					* ((2.0 * Math.PI) / (double) triesPerCell);
+			double angle = i
+					* ((2.0 * Math.PI) / triesPerCell);
 			xNormTry[i] = Math.cos(angle);
 			yNormTry[i] = Math.sin(angle);
 		}
@@ -991,7 +991,7 @@ public class mxOrganicLayout extends mxGraphLayout
 				}
 			}
 		}
-		return edgeCrossingCostFactor * (double) n;
+		return edgeCrossingCostFactor * n;
 	}
 
 	/**
