@@ -1,5 +1,5 @@
 /**
- * $Id: mxGraphModel.java,v 1.78 2010/01/13 10:43:46 gaudenz Exp $
+ * $Id: mxGraphModel.java,v 1.82 2010-08-02 13:49:12 david Exp $
  * Copyright (c) 2007, Gaudenz Alder
  */
 package com.mxgraph.model;
@@ -757,8 +757,6 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 					setGeometry(edge, geo);
 				}
 
-				System.out.println("Moving edge " + ((mxICell) edge).getValue()
-						+ " to parent " + mxCellPath.create((mxICell) cell));
 				add(cell, edge, getChildCount(cell));
 			}
 		}
@@ -804,8 +802,6 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 	 */
 	public Object getNearestCommonAncestor(Object cell1, Object cell2)
 	{
-		Object result = null;
-
 		if (cell1 != null && cell2 != null)
 		{
 			// Creates the cell path for the second cell
@@ -818,7 +814,7 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 				Object cell = cell1;
 				String current = mxCellPath.create((mxICell) cell);
 
-				while (cell != null && result == null)
+				while (cell != null)
 				{
 					Object parent = getParent(cell);
 
@@ -827,7 +823,7 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 					if (path.indexOf(current + mxCellPath.PATH_SEPARATOR) == 0
 							&& parent != null)
 					{
-						result = cell;
+						return cell;
 					}
 
 					current = mxCellPath.getParentPath(current);
@@ -836,7 +832,7 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			}
 		}
 
-		return result;
+		return null;
 	}
 
 	/* (non-Javadoc)
@@ -1192,7 +1188,7 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 						// Do *NOT* use model.add as this will move the edge away
 						// from the parent in updateEdgeParent if maintainEdgeParent
 						// is enabled in the target model
-						target = (mxICell) to.insert(clone);
+						target = to.insert(clone);
 						cellAdded(target);
 					}
 
@@ -1764,11 +1760,6 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		 */
 		protected int index, previousIndex;
 
-		/**
-		 * 
-		 */
-		protected boolean added;
-
 		/**
 		 * 
 		 */
@@ -1789,7 +1780,6 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			this.child = child;
 			this.index = index;
 			previousIndex = index;
-			added = (parent == null);
 		}
 
 		/**
@@ -1832,14 +1822,6 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return previousIndex;
 		}
 
-		/**
-		 * @return the isAdded
-		 */
-		public boolean isAdded()
-		{
-			return added;
-		}
-
 		/**
 		 * Gets the source or target terminal field for the given
 		 * edge even if the edge is not stored as an incoming or
@@ -1904,7 +1886,7 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 
 			for (int i = 0; i < childCount; i++)
 			{
-				connect((mxICell) model.getChildAt(cell, i), isConnect);
+				connect(model.getChildAt(cell, i), isConnect);
 			}
 		}
 
@@ -1943,7 +1925,6 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			previous = tmp;
 			index = previousIndex;
 			previousIndex = tmp2;
-			added = !added;
 		}
 
 	}
