@@ -1,5 +1,5 @@
 /**
- * $Id: mxEdgeHandler.java,v 1.19 2010/01/05 09:05:25 gaudenz Exp $
+ * $Id: mxEdgeHandler.java,v 1.25 2010-06-01 10:18:45 gaudenz Exp $
  * Copyright (c) 2008, Gaudenz Alder
  */
 package com.mxgraph.swing.handler;
@@ -12,6 +12,7 @@ import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.Stroke;
 import java.awt.event.MouseEvent;
+import java.awt.geom.Line2D;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
@@ -26,7 +27,9 @@ import com.mxgraph.swing.mxGraphComponent;
 import com.mxgraph.util.mxConstants;
 import com.mxgraph.util.mxPoint;
 import com.mxgraph.view.mxCellState;
+import com.mxgraph.view.mxConnectionConstraint;
 import com.mxgraph.view.mxGraph;
+import com.mxgraph.view.mxGraphView;
 
 /**
  *
@@ -202,7 +205,7 @@ public class mxEdgeHandler extends mxCellHandler
 		}
 
 		h[p.length] = createHandle(state.getAbsoluteOffset().getPoint(),
-				mxConstants.HANDLE_SIZE - 2);
+				mxConstants.LABEL_HANDLE_SIZE);
 
 		return h;
 	}
@@ -356,7 +359,6 @@ public class mxEdgeHandler extends mxCellHandler
 		mxGraph graph = graphComponent.getGraph();
 		double scale = graph.getView().getScale();
 		mxPoint trans = graph.getView().getTranslate();
-
 		double x = point.getX() / scale - trans.getX();
 		double y = point.getY() / scale - trans.getY();
 
@@ -433,6 +435,8 @@ public class mxEdgeHandler extends mxCellHandler
 			constrainedEvent = graphComponent.isConstrainedEvent(e);
 
 			boolean isSource = isSource(index);
+			boolean isTarget = isTarget(index);
+
 			Object source = null;
 			Object target = null;
 
@@ -476,14 +480,15 @@ public class mxEdgeHandler extends mxCellHandler
 						.getCellGeometry(state.getCell());
 				mxCellState clone = (mxCellState) state.clone();
 				List<mxPoint> points = geometry.getPoints();
+				mxGraphView view = clone.getView();
 
-				if (isSource || isTarget(index))
+				if (isSource || isTarget)
 				{
 					marker.process(e);
 					mxCellState currentState = marker.getValidState();
 
-					target = clone.getView().getVisibleTerminal(
-							state.getCell(), !isSource);
+					target = view
+							.getVisibleTerminal(state.getCell(), !isSource);
 
 					if (currentState != null)
 					{
@@ -523,15 +528,54 @@ public class mxEdgeHandler extends mxCellHandler
 						points.set(index - 1, point);
 					}
 
-					source = clone.getView().getVisibleTerminal(
-							state.getCell(), true);
-					target = clone.getView().getVisibleTerminal(
-							state.getCell(), false);
+					source = view.getVisibleTerminal(state.getCell(), true);
+					target = view.getVisibleTerminal(state.getCell(), false);
 				}
 
 				// Computes the points for the edge style and terminals
-				clone.getView().updatePoints(clone, points, source, target);
-				clone.getView().updateTerminalPoints(clone, source, target);
+				mxCellState sourceState = view.getState(source);
+				mxCellState targetState = view.getState(target);
+
+				mxConnectionConstraint sourceConstraint = graphComponent
+						.getGraph().getConnectionConstraint(clone, sourceState,
+								true);
+				mxConnectionConstraint targetConstraint = graphComponent
+						.getGraph().getConnectionConstraint(clone, targetState,
+								false);
+
+				/* TODO: Implement mxConstraintHandler
+				mxConnectionConstraint constraint = constraintHandler.currentConstraint;
+
+				if (constraint == null)
+				{
+					constraint = new mxConnectionConstraint();
+				}
+				
+				if (isSource)
+				{
+					sourceConstraint = constraint;
+				}
+				else if (isTarget)
+				{
+					targetConstraint = constraint;
+				}
+				*/
+
+				if (!isSource || sourceState != null)
+				{
+					view.updateFixedTerminalPoint(clone, sourceState, true,
+							sourceConstraint);
+				}
+
+				if (!isTarget || targetState != null)
+				{
+					view.updateFixedTerminalPoint(clone, targetState, false,
+							targetConstraint);
+				}
+
+				view.updatePoints(clone, points, sourceState, targetState);
+				view.updateFloatingTerminalPoints(clone, sourceState,
+						targetState);
 
 				// Uses the updated points from the cloned state to draw the preview
 				p = createPoints(clone);
@@ -607,7 +651,7 @@ public class mxEdgeHandler extends mxCellHandler
 						&& (isSource(index) || isTarget(index)))
 				{
 					connect(state.getCell(), marker.getValidState().getCell(),
-							isSource(index), e.isControlDown()
+							isSource(index), graphComponent.isCloneEvent(e)
 									&& isCloneEnabled());
 				}
 				else if ((!isSource(index) && !isTarget(index))
@@ -627,10 +671,18 @@ public class mxEdgeHandler extends mxCellHandler
 			e.consume();
 		}
 
+		super.mouseReleased(e);
+	}
+
+	/**
+	 * Extends the implementation to reset the current error and marker.
+	 */
+	public void reset()
+	{
+		super.reset();
+
 		marker.reset();
 		error = null;
-
-		super.mouseReleased(e);
 	}
 
 	/**
@@ -715,7 +767,9 @@ public class mxEdgeHandler extends mxCellHandler
 				edge = clone;
 			}
 
-			graph.connectCell(edge, terminal, isSource);
+			// Passes an empty constraint to reset constraint information
+			graph.connectCell(edge, terminal, isSource,
+					new mxConnectionConstraint());
 		}
 		finally
 		{
@@ -760,11 +814,7 @@ public class mxEdgeHandler extends mxCellHandler
 	{
 		Cursor cursor = null;
 
-		if (isSource(index) || isTarget(index))
-		{
-			cursor = mxConnectionHandler.DEFAULT_CURSOR;
-		}
-		else if (isLabel(index))
+		if (isLabel(index))
 		{
 			cursor = new Cursor(Cursor.MOVE_CURSOR);
 		}
@@ -808,7 +858,15 @@ public class mxEdgeHandler extends mxCellHandler
 		for (int i = 1; i < state.getAbsolutePointCount(); i++)
 		{
 			Point current = state.getAbsolutePoint(i).getPoint();
-			g.drawLine(last.x, last.y, current.x, current.y);
+			Line2D line = new Line2D.Float(last.x, last.y, current.x, current.y);
+			
+			Rectangle bounds = g2.getStroke().createStrokedShape(line).getBounds();
+
+			if (g.hitClip(bounds.x, bounds.y, bounds.width, bounds.height))
+			{
+				g2.draw(line);
+			}
+			
 			last = current;
 		}
 
