@@ -1,5 +1,5 @@
 /**
- * $Id: mxGraphHandler.java,v 1.50 2010/01/06 09:24:36 gaudenz Exp $
+ * $Id: mxGraphHandler.java,v 1.70 2010-08-23 15:19:23 gaudenz Exp $
  * Copyright (c) 2008, Gaudenz Alder
  * 
  * Known issue: Drag image size depends on the initial position and may sometimes
@@ -9,9 +9,11 @@
  */
 package com.mxgraph.swing.handler;
 
+import java.awt.AlphaComposite;
 import java.awt.Color;
 import java.awt.Cursor;
 import java.awt.Graphics;
+import java.awt.Graphics2D;
 import java.awt.Image;
 import java.awt.Point;
 import java.awt.Rectangle;
@@ -37,17 +39,19 @@ import javax.swing.SwingUtilities;
 import javax.swing.TransferHandler;
 
 import com.mxgraph.swing.mxGraphComponent;
-import com.mxgraph.swing.mxGraphComponent.mxMouseRedirector;
 import com.mxgraph.swing.util.mxGraphTransferable;
-import com.mxgraph.swing.util.mxMouseControl;
+import com.mxgraph.swing.util.mxMouseAdapter;
 import com.mxgraph.util.mxCellRenderer;
 import com.mxgraph.util.mxConstants;
+import com.mxgraph.util.mxEvent;
+import com.mxgraph.util.mxEventObject;
 import com.mxgraph.util.mxPoint;
 import com.mxgraph.util.mxRectangle;
+import com.mxgraph.util.mxEventSource.mxIEventListener;
 import com.mxgraph.view.mxCellState;
 import com.mxgraph.view.mxGraph;
 
-public class mxGraphHandler extends mxMouseControl implements
+public class mxGraphHandler extends mxMouseAdapter implements
 		DropTargetListener
 {
 
@@ -57,15 +61,30 @@ public class mxGraphHandler extends mxMouseControl implements
 	private static final long serialVersionUID = 3241109976696510225L;
 
 	/**
-	 * 
+	 * Default is Cursor.DEFAULT_CURSOR.
+	 */
+	public static Cursor DEFAULT_CURSOR = new Cursor(Cursor.DEFAULT_CURSOR);
+
+	/**
+	 * Default is Cursor.MOVE_CURSOR.
 	 */
 	public static Cursor MOVE_CURSOR = new Cursor(Cursor.MOVE_CURSOR);
 
+	/**
+	 * Default is Cursor.HAND_CURSOR.
+	 */
+	public static Cursor FOLD_CURSOR = new Cursor(Cursor.HAND_CURSOR);
+
 	/**
 	 * Reference to the enclosing graph component.
 	 */
 	protected mxGraphComponent graphComponent;
 
+	/**
+	 * Specifies if the handler is enabled. Default is true.
+	 */
+	protected boolean enabled = true;
+
 	/**
 	 * Specifies if cloning by control-drag is enabled. Default is true.
 	 */
@@ -82,10 +101,26 @@ public class mxGraphHandler extends mxMouseControl implements
 	protected boolean selectEnabled = true;
 
 	/**
-	 * Specifies if cells may be moved out of their parents.
+	 * Specifies if the cell marker should be called (for splitting edges and
+	 * dropping cells into groups). Default is true.
+	 */
+	protected boolean markerEnabled = true;
+
+	/**
+	 * Specifies if cells may be moved out of their parents. Default is true.
 	 */
 	protected boolean removeCellsFromParent = true;
 
+	/**
+	 * 
+	 */
+	protected mxMovePreview movePreview;
+
+	/**
+	 * Specifies if live preview should be used if possible. Default is false.
+	 */
+	protected boolean livePreview = false;
+
 	/**
 	 * Specifies if an image should be used for preview. Default is true.
 	 */
@@ -159,6 +194,16 @@ public class mxGraphHandler extends mxMouseControl implements
 	 */
 	protected transient mxRectangle transferBounds;
 
+	/**
+	 * 
+	 */
+	protected transient boolean visible = false;
+
+	/**
+	 * 
+	 */
+	protected transient Rectangle previewBounds = null;
+
 	/**
 	 * Workaround for alt-key-state not correct in mouseReleased. Note: State
 	 * of the alt-key is not available during drag-and-drop.
@@ -178,23 +223,36 @@ public class mxGraphHandler extends mxMouseControl implements
 	{
 		this.graphComponent = graphComponent;
 		marker = createMarker();
+		movePreview = createMovePreview();
 
-		// Adds component for rendering the handles (preview is separate)
-		graphComponent.getGraphControl().add(this, 0);
+		// Installs the paint handler
+		graphComponent.addListener(mxEvent.AFTER_PAINT, new mxIEventListener()
+		{
+			public void invoke(Object sender, mxEventObject evt)
+			{
+				Graphics g = (Graphics) evt.getProperty("g");
+				paint(g);
+			}
+		});
 
 		// Listens to all mouse events on the rendering control
 		graphComponent.getGraphControl().addMouseListener(this);
 		graphComponent.getGraphControl().addMouseMotionListener(this);
 
-		// Redirects events from component to rendering control so
-		// that the event handling order is maintained if other controls
-		// such as overlays are added to the component hierarchy and
-		// consume events before they reach the rendering control
-		mxMouseRedirector redirector = new mxMouseRedirector(graphComponent);
-		addMouseMotionListener(redirector);
-		addMouseListener(redirector);
-
 		// Drag target creates preview image
+		installDragGestureHandler();
+
+		// Listens to dropped graph cells
+		installDropTargetHandler();
+
+		setVisible(false);
+	}
+
+	/**
+	 * 
+	 */
+	protected void installDragGestureHandler()
+	{
 		DragGestureListener dragGestureListener = new DragGestureListener()
 		{
 			public void dragGestureRecognized(DragGestureEvent e)
@@ -236,11 +294,16 @@ public class mxGraphHandler extends mxMouseControl implements
 		};
 
 		DragSource dragSource = new DragSource();
-		dragSource.createDefaultDragGestureRecognizer(graphComponent
-				.getGraphControl(), DnDConstants.ACTION_COPY_OR_MOVE,
-				dragGestureListener);
+		dragSource.createDefaultDragGestureRecognizer(
+				graphComponent.getGraphControl(),
+				DnDConstants.ACTION_COPY_OR_MOVE, dragGestureListener);
+	}
 
-		// Listens to dropped graph cells
+	/**
+	 * 
+	 */
+	protected void installDropTargetHandler()
+	{
 		DropTarget dropTarget = graphComponent.getDropTarget();
 
 		try
@@ -254,10 +317,74 @@ public class mxGraphHandler extends mxMouseControl implements
 		{
 			// should not happen... swing drop target is multicast
 		}
+	}
 
-		setOpaque(false);
-		setVisible(false);
-		setBorder(mxConstants.PREVIEW_BORDER);
+	/**
+	 * 
+	 */
+	public boolean isVisible()
+	{
+		return visible;
+	}
+
+	/**
+	 * 
+	 */
+	public void setVisible(boolean value)
+	{
+		if (visible != value)
+		{
+			visible = value;
+
+			if (previewBounds != null)
+			{
+				graphComponent.getGraphControl().repaint(previewBounds);
+			}
+		}
+	}
+
+	/**
+	 * 
+	 */
+	public void setPreviewBounds(Rectangle bounds)
+	{
+		if ((bounds == null && previewBounds != null)
+				|| (bounds != null && previewBounds == null)
+				|| (bounds != null && previewBounds != null && !bounds
+						.equals(previewBounds)))
+		{
+			Rectangle dirty = null;
+
+			if (isVisible())
+			{
+				dirty = previewBounds;
+
+				if (dirty != null)
+				{
+					dirty.add(bounds);
+				}
+				else
+				{
+					dirty = bounds;
+				}
+			}
+
+			previewBounds = bounds;
+
+			if (dirty != null)
+			{
+				graphComponent.getGraphControl().repaint(dirty.x - 1,
+						dirty.y - 1, dirty.width + 2, dirty.height + 2);
+			}
+		}
+	}
+
+	/**
+	 * 
+	 */
+	protected mxMovePreview createMovePreview()
+	{
+		return new mxMovePreview(graphComponent);
 	}
 
 	/**
@@ -321,6 +448,22 @@ public class mxGraphHandler extends mxMouseControl implements
 		return graphComponent;
 	}
 
+	/**
+	 * 
+	 */
+	public boolean isEnabled()
+	{
+		return enabled;
+	}
+
+	/**
+	 * 
+	 */
+	public void setEnabled(boolean value)
+	{
+		enabled = value;
+	}
+
 	/**
 	 * 
 	 */
@@ -353,6 +496,22 @@ public class mxGraphHandler extends mxMouseControl implements
 		moveEnabled = value;
 	}
 
+	/**
+	 * 
+	 */
+	public boolean isMarkerEnabled()
+	{
+		return markerEnabled;
+	}
+
+	/**
+	 * 
+	 */
+	public void setMarkerEnabled(boolean value)
+	{
+		markerEnabled = value;
+	}
+
 	/**
 	 * 
 	 */
@@ -385,6 +544,22 @@ public class mxGraphHandler extends mxMouseControl implements
 		removeCellsFromParent = value;
 	}
 
+	/**
+	 * 
+	 */
+	public boolean isLivePreview()
+	{
+		return livePreview;
+	}
+
+	/**
+	 * 
+	 */
+	public void setLivePreview(boolean value)
+	{
+		livePreview = value;
+	}
+
 	/**
 	 * 
 	 */
@@ -422,17 +597,22 @@ public class mxGraphHandler extends mxMouseControl implements
 	 */
 	public void updateDragImage(Object[] cells)
 	{
-		Image img = mxCellRenderer.createBufferedImage(graphComponent
-				.getGraph(), cells, graphComponent.getGraph().getView()
-				.getScale(), null, graphComponent.isAntiAlias(), null,
-				graphComponent.getCanvas());
+		dragImage = null;
 
-		if (img != null)
+		if (cells != null && cells.length > 0)
 		{
-			dragImage = new ImageIcon(img);
+			Image img = mxCellRenderer.createBufferedImage(
+					graphComponent.getGraph(), cells, graphComponent.getGraph()
+							.getView().getScale(), null,
+					graphComponent.isAntiAlias(), null,
+					graphComponent.getCanvas());
 
-			setSize(dragImage.getIconWidth(), dragImage.getIconHeight());
-			getParent().setComponentZOrder(this, 0);
+			if (img != null)
+			{
+				dragImage = new ImageIcon(img);
+				previewBounds.setSize(dragImage.getIconWidth(),
+						dragImage.getIconHeight());
+			}
 		}
 	}
 
@@ -452,8 +632,7 @@ public class mxGraphHandler extends mxMouseControl implements
 			}
 			else
 			{
-				graphComponent.getGraphControl().setCursor(
-						new Cursor(Cursor.DEFAULT_CURSOR));
+				graphComponent.getGraphControl().setCursor(DEFAULT_CURSOR);
 			}
 		}
 	}
@@ -472,10 +651,10 @@ public class mxGraphHandler extends mxMouseControl implements
 			if (cell != null)
 			{
 				if (graphComponent.isFoldingEnabled()
-						&& graphComponent.hitFoldingIcon(cell, e.getX(), e
-								.getY()))
+						&& graphComponent.hitFoldingIcon(cell, e.getX(),
+								e.getY()))
 				{
-					cursor = new Cursor(Cursor.HAND_CURSOR);
+					cursor = FOLD_CURSOR;
 				}
 				else if (graphComponent.getGraph().isCellMovable(cell))
 				{
@@ -509,7 +688,6 @@ public class mxGraphHandler extends mxMouseControl implements
 
 		if (canImport)
 		{
-			e.acceptDrag(TransferHandler.COPY_OR_MOVE);
 			transferBounds = null;
 			setVisible(false);
 
@@ -527,12 +705,14 @@ public class mxGraphHandler extends mxMouseControl implements
 					{
 						mxGraph graph = graphComponent.getGraph();
 						double scale = graph.getView().getScale();
-
 						transferBounds = gt.getBounds();
-						mxRectangle bounds = new mxRectangle(transferBounds);
-						bounds.setWidth(bounds.getWidth() * scale + 1);
-						bounds.setHeight(bounds.getHeight() * scale + 1);
-						setBounds(bounds.getRectangle());
+						int w = (int) Math.ceil((transferBounds.getWidth() + 1)
+								* scale);
+						int h = (int) Math
+								.ceil((transferBounds.getHeight() + 1) * scale);
+						setPreviewBounds(new Rectangle(
+								(int) transferBounds.getX(),
+								(int) transferBounds.getY(), w, h));
 
 						if (imagePreview)
 						{
@@ -540,19 +720,35 @@ public class mxGraphHandler extends mxMouseControl implements
 							// but ignores movable state for non-local previews
 							if (isLocal)
 							{
-								updateDragImage(graph
-										.getMovableCells(dragCells));
+								if (!isLivePreview())
+								{
+									updateDragImage(graph
+											.getMovableCells(dragCells));
+								}
 							}
 							else
 							{
-								updateDragImage(graphComponent
-										.getImportableCells(dragCells));
+								Object[] tmp = graphComponent
+										.getImportableCells(dragCells);
+								updateDragImage(tmp);
+
+								// Shows no drag icon if import is allowed but none
+								// of the cells can be imported
+								if (tmp == null || tmp.length == 0)
+								{
+									canImport = false;
+									e.rejectDrag();
+
+									return;
+								}
 							}
 						}
 
 						setVisible(true);
 					}
 				}
+
+				e.acceptDrag(TransferHandler.COPY_OR_MOVE);
 			}
 			catch (Exception ex)
 			{
@@ -581,8 +777,7 @@ public class mxGraphHandler extends mxMouseControl implements
 			if (cell != null)
 			{
 				if (isSelectEnabled()
-						&& !graphComponent.getGraph().isCellSelected(cell)
-						&& !graphComponent.isToggleEvent(e))
+						&& !graphComponent.getGraph().isCellSelected(cell))
 				{
 					graphComponent.selectCellForEvent(cell, e);
 					cell = null;
@@ -618,25 +813,30 @@ public class mxGraphHandler extends mxMouseControl implements
 	 */
 	public void start(MouseEvent e)
 	{
-		mxGraph graph = graphComponent.getGraph();
-
-		// Constructs an array with cells that are indeed movable
-		cells = getCells(initialCell);
-		cellBounds = graph.getView().getBounds(cells);
-
-		if (cellBounds != null)
+		if (isLivePreview())
 		{
-			// Keeps this handler in the foreground
-			getParent().setComponentZOrder(this, 0);
+			movePreview.start(e,
+					graphComponent.getGraph().getView().getState(initialCell));
+		}
+		else
+		{
+			mxGraph graph = graphComponent.getGraph();
 
-			// Updates the size of the graph handler that is in
-			// charge of painting all other handlers
-			bbox = graph.getView().getBoundingBox(cells);
+			// Constructs an array with cells that are indeed movable
+			cells = getCells(initialCell);
+			cellBounds = graph.getView().getBounds(cells);
 
-			Rectangle bounds = cellBounds.getRectangle();
-			bounds.width += 1;
-			bounds.height += 1;
-			setBounds(bounds);
+			if (cellBounds != null)
+			{
+				// Updates the size of the graph handler that is in
+				// charge of painting all other handlers
+				bbox = graph.getView().getBoundingBox(cells);
+
+				Rectangle bounds = cellBounds.getRectangle();
+				bounds.width += 1;
+				bounds.height += 1;
+				setPreviewBounds(bounds);
+			}
 		}
 
 		first = e.getPoint();
@@ -665,8 +865,8 @@ public class mxGraphHandler extends mxMouseControl implements
 			{
 				mxGraph graph = graphComponent.getGraph();
 				double scale = graph.getView().getScale();
-				Point pt = SwingUtilities.convertPoint(graphComponent, e
-						.getLocation(), graphComponent.getGraphControl());
+				Point pt = SwingUtilities.convertPoint(graphComponent,
+						e.getLocation(), graphComponent.getGraphControl());
 
 				pt = graphComponent.snapScaledPoint(new mxPoint(pt)).getPoint();
 				handler.setLocation(new Point(pt));
@@ -677,10 +877,8 @@ public class mxGraphHandler extends mxMouseControl implements
 				// Centers the preview image
 				if (centerPreview && transferBounds != null)
 				{
-					dx -= Math.round((double) transferBounds.getWidth() * scale
-							/ 2);
-					dy -= Math.round((double) transferBounds.getHeight()
-							* scale / 2);
+					dx -= Math.round(transferBounds.getWidth() * scale / 2);
+					dy -= Math.round(transferBounds.getHeight() * scale / 2);
 				}
 
 				// Sets the drop offset so that the location in the transfer
@@ -695,18 +893,16 @@ public class mxGraphHandler extends mxMouseControl implements
 				{
 					dx = (int) Math
 							.round((dragImage.getIconWidth() - 2 - transferBounds
-									.getWidth()
-									* scale) / 2);
+									.getWidth() * scale) / 2);
 					dy = (int) Math
 							.round((dragImage.getIconHeight() - 2 - transferBounds
-									.getHeight()
-									* scale) / 2);
+									.getHeight() * scale) / 2);
 					pt.translate(-dx, -dy);
 				}
 
-				if (!handler.isLocalDrag())
+				if (!handler.isLocalDrag() && previewBounds != null)
 				{
-					setLocation(pt.x, pt.y);
+					setPreviewBounds(new Rectangle(pt, previewBounds.getSize()));
 				}
 			}
 		}
@@ -721,8 +917,8 @@ public class mxGraphHandler extends mxMouseControl implements
 	 */
 	public Point convertPoint(Point pt)
 	{
-		pt = SwingUtilities.convertPoint(graphComponent, pt, graphComponent
-				.getGraphControl());
+		pt = SwingUtilities.convertPoint(graphComponent, pt,
+				graphComponent.getGraphControl());
 
 		pt.x -= graphComponent.getHorizontalScrollBar().getValue();
 		pt.y -= graphComponent.getVerticalScrollBar().getValue();
@@ -759,42 +955,60 @@ public class mxGraphHandler extends mxMouseControl implements
 					x = first.x;
 				}
 
-				e = new MouseEvent(e.getComponent(), e.getID(), e.getWhen(), e
-						.getModifiers(), x, y, e.getClickCount(), e
-						.isPopupTrigger(), e.getButton());
+				e = new MouseEvent(e.getComponent(), e.getID(), e.getWhen(),
+						e.getModifiers(), x, y, e.getClickCount(),
+						e.isPopupTrigger(), e.getButton());
 			}
 
-			if (isVisible())
+			if (isVisible() && isMarkerEnabled())
 			{
 				marker.process(e);
 			}
-			else if (cell != null && isSelectEnabled()
-					&& !graphComponent.getGraph().isCellSelected(cell)
-					&& graphComponent.isToggleEvent(e))
+
+			if (first != null)
 			{
-				graphComponent.selectCellForEvent(cell, e);
-				cell = null;
-			}
-
-			if (first != null && cellBounds != null)
-			{
-				double dx = e.getX() - first.x;
-				double dy = e.getY() - first.y;
-
-				setLocation(getPreviewLocation(e, gridEnabledEvent));
-
-				if (!isVisible() && graphComponent.isSignificant(dx, dy))
+				if (movePreview.isActive())
 				{
-					if (imagePreview && dragImage == null
-							&& !graphComponent.isDragEnabled())
+					double dx = e.getX() - first.x;
+					double dy = e.getY() - first.y;
+
+					if (graphComponent.isGridEnabledEvent(e))
 					{
-						updateDragImage(cells);
+						mxGraph graph = graphComponent.getGraph();
+
+						dx = graph.snap(dx);
+						dy = graph.snap(dy);
 					}
 
-					setVisible(true);
+					boolean clone = isCloneEnabled()
+							&& graphComponent.isCloneEvent(e);
+					movePreview.update(e, dx, dy, clone);
+					e.consume();
 				}
+				else if (cellBounds != null)
+				{
+					double dx = e.getX() - first.x;
+					double dy = e.getY() - first.y;
 
-				e.consume();
+					if (previewBounds != null)
+					{
+						setPreviewBounds(new Rectangle(getPreviewLocation(e,
+								gridEnabledEvent), previewBounds.getSize()));
+					}
+
+					if (!isVisible() && graphComponent.isSignificant(dx, dy))
+					{
+						if (imagePreview && dragImage == null
+								&& !graphComponent.isDragEnabled())
+						{
+							updateDragImage(cells);
+						}
+
+						setVisible(true);
+					}
+
+					e.consume();
+				}
 			}
 		}
 	}
@@ -857,6 +1071,7 @@ public class mxGraphHandler extends mxMouseControl implements
 		dragCells = null;
 		setVisible(false);
 		marker.reset();
+		reset();
 	}
 
 	/**
@@ -892,7 +1107,7 @@ public class mxGraphHandler extends mxMouseControl implements
 			double dx = 0;
 			double dy = 0;
 
-			if (first != null && cellBounds != null)
+			if (first != null && (cellBounds != null || movePreview.isActive()))
 			{
 				double scale = graph.getView().getScale();
 				mxPoint trans = graph.getView().getTranslate();
@@ -902,58 +1117,32 @@ public class mxGraphHandler extends mxMouseControl implements
 				dx = e.getX() - first.x;
 				dy = e.getY() - first.y;
 
-				double dxg = ((cellBounds.getX() + dx) / scale) - trans.getX();
-				double dyg = ((cellBounds.getY() + dy) / scale) - trans.getY();
-
-				if (gridEnabledEvent)
+				if (cellBounds != null)
 				{
-					dxg = graph.snap(dxg);
-					dyg = graph.snap(dyg);
+					double dxg = ((cellBounds.getX() + dx) / scale)
+							- trans.getX();
+					double dyg = ((cellBounds.getY() + dy) / scale)
+							- trans.getY();
+
+					if (gridEnabledEvent)
+					{
+						dxg = graph.snap(dxg);
+						dyg = graph.snap(dyg);
+					}
+
+					double x = ((dxg + trans.getX()) * scale) + (bbox.getX())
+							- (cellBounds.getX());
+					double y = ((dyg + trans.getY()) * scale) + (bbox.getY())
+							- (cellBounds.getY());
+
+					dx = Math.round((x - bbox.getX()) / scale);
+					dy = Math.round((y - bbox.getY()) / scale);
 				}
-
-				double x = ((dxg + trans.getX()) * scale) + (bbox.getX())
-						- (cellBounds.getX());
-				double y = ((dyg + trans.getY()) * scale) + (bbox.getY())
-						- (cellBounds.getY());
-
-				dx = Math.round((x - bbox.getX()) / scale);
-				dy = Math.round((y - bbox.getY()) / scale);
 			}
 
-			if (isVisible())
-			{
-				if (constrainedEvent)
-				{
-					if (Math.abs(dx) > Math.abs(dy))
-					{
-						dy = 0;
-					}
-					else
-					{
-						dx = 0;
-					}
-				}
-
-				mxCellState targetState = marker.getValidState();
-				Object target = (targetState != null) ? targetState.getCell()
-						: null;
-
-				if (graph.isSplitEnabled()
-						&& graph.isSplitTarget(target, cells))
-				{
-					graph.splitEdge(target, cells, dx, dy);
-				}
-				else
-				{
-					moveCells(cells, dx, dy, target, e);
-				}
-
-				e.consume();
-			}
-			else if (first == null
-					|| !graphComponent.isSignificant(e.getX() - first.x, e
-							.getY()
-							- first.y))
+			if (first == null
+					|| !graphComponent.isSignificant(e.getX() - first.x,
+							e.getY() - first.y))
 			{
 				// Delayed handling of selection
 				if (cell != null && !e.isPopupTrigger() && isSelectEnabled()
@@ -992,14 +1181,85 @@ public class mxGraphHandler extends mxMouseControl implements
 					}
 
 					if (graphComponent.isFoldingEnabled()
-							&& graphComponent.hitFoldingIcon(tmp, e.getX(), e
-									.getY()))
+							&& graphComponent.hitFoldingIcon(tmp, e.getX(),
+									e.getY()))
 					{
 						fold(tmp);
 						e.consume();
 					}
 				}
 			}
+			else if (movePreview.isActive())
+			{
+				if (graphComponent.isConstrainedEvent(e))
+				{
+					if (Math.abs(dx) > Math.abs(dy))
+					{
+						dy = 0;
+					}
+					else
+					{
+						dx = 0;
+					}
+				}
+
+				mxCellState markedState = marker.getMarkedState();
+				Object target = (markedState != null) ? markedState.getCell()
+						: null;
+
+				// FIXME: Cell is null if selection was carried out, need other variable
+				//trace("cell", cell);
+
+				if (target == null
+						&& isRemoveCellsFromParent()
+						&& shouldRemoveCellFromParent(graph.getModel()
+								.getParent(initialCell), cells, e))
+				{
+					target = graph.getDefaultParent();
+				}
+
+				boolean clone = isCloneEnabled()
+						&& graphComponent.isCloneEvent(e);
+				Object[] result = movePreview.stop(true, e, dx, dy, clone,
+						target);
+
+				if (cells != result)
+				{
+					graph.setSelectionCells(result);
+				}
+
+				e.consume();
+			}
+			else if (isVisible())
+			{
+				if (constrainedEvent)
+				{
+					if (Math.abs(dx) > Math.abs(dy))
+					{
+						dy = 0;
+					}
+					else
+					{
+						dx = 0;
+					}
+				}
+
+				mxCellState targetState = marker.getValidState();
+				Object target = (targetState != null) ? targetState.getCell()
+						: null;
+
+				if (graph.isSplitEnabled()
+						&& graph.isSplitTarget(target, cells))
+				{
+					graph.splitEdge(target, cells, dx, dy);
+				}
+				else
+				{
+					moveCells(cells, dx, dy, target, e);
+				}
+
+				e.consume();
+			}
 		}
 
 		reset();
@@ -1020,6 +1280,11 @@ public class mxGraphHandler extends mxMouseControl implements
 	 */
 	public void reset()
 	{
+		if (movePreview.isActive())
+		{
+			movePreview.stop(false, null, 0, 0, false, null);
+		}
+
 		setVisible(false);
 		marker.reset();
 		initialCell = null;
@@ -1039,8 +1304,8 @@ public class mxGraphHandler extends mxMouseControl implements
 	{
 		if (graphComponent.getGraph().getModel().isVertex(parent))
 		{
-			mxCellState pState = graphComponent.getGraph().getView().getState(
-					parent);
+			mxCellState pState = graphComponent.getGraph().getView()
+					.getState(parent);
 
 			return pState != null && !pState.contains(e.getX(), e.getY());
 		}
@@ -1068,14 +1333,14 @@ public class mxGraphHandler extends mxMouseControl implements
 		// Removes cells from parent
 		if (target == null
 				&& isRemoveCellsFromParent()
-				&& shouldRemoveCellFromParent(graph.getModel().getParent(cell),
-						cells, e))
+				&& shouldRemoveCellFromParent(
+						graph.getModel().getParent(initialCell), cells, e))
 		{
 			target = graph.getDefaultParent();
 		}
 
-		Object[] tmp = graph.moveCells(cells, dx, dy, clone, target, e
-				.getPoint());
+		Object[] tmp = graph.moveCells(cells, dx, dy, clone, target,
+				e.getPoint());
 
 		if (isSelectEnabled() && clone && tmp != null
 				&& tmp.length == cells.length)
@@ -1085,20 +1350,36 @@ public class mxGraphHandler extends mxMouseControl implements
 	}
 
 	/**
-	 * 
+	 *
 	 */
 	public void paint(Graphics g)
 	{
-		if (dragImage != null)
+		if (isVisible() && previewBounds != null)
 		{
-			// LATER: Clipping with mxUtils doesnt fix the problem
-			// of the drawImage being painted over the scrollbars
-			g.drawImage(dragImage.getImage(), 0, 0, dragImage.getIconWidth(),
-					dragImage.getIconHeight(), this);
-		}
-		else if (!imagePreview)
-		{
-			super.paint(g);
+			if (dragImage != null)
+			{
+				// LATER: Clipping with mxUtils doesnt fix the problem
+				// of the drawImage being painted over the scrollbars
+				Graphics2D tmp = (Graphics2D) g.create();
+
+				if (graphComponent.getPreviewAlpha() < 1)
+				{
+					tmp.setComposite(AlphaComposite.getInstance(
+							AlphaComposite.SRC_OVER,
+							graphComponent.getPreviewAlpha()));
+				}
+
+				tmp.drawImage(dragImage.getImage(), previewBounds.x,
+						previewBounds.y, dragImage.getIconWidth(),
+						dragImage.getIconHeight(), null);
+				tmp.dispose();
+			}
+			else if (!imagePreview)
+			{
+				mxConstants.PREVIEW_BORDER.paintBorder(graphComponent, g,
+						previewBounds.x, previewBounds.y, previewBounds.width,
+						previewBounds.height);
+			}
 		}
 	}
 
@@ -1137,8 +1418,7 @@ public class mxGraphHandler extends mxMouseControl implements
 		int mod = (action == TransferHandler.COPY) ? InputEvent.CTRL_MASK : 0;
 
 		return new MouseEvent(component, 0, System.currentTimeMillis(), mod,
-				(int) location.getX(), (int) location.getY(), 1, false,
-				MouseEvent.BUTTON1);
+				location.x, location.y, 1, false, MouseEvent.BUTTON1);
 	}
 
 	/**
