@@ -1,5 +1,5 @@
 /**
- * $Id: mxGraphOutline.java,v 1.20 2009/11/29 13:48:01 gaudenz Exp $
+ * $Id: mxGraphOutline.java,v 1.22 2010-05-31 08:49:27 gaudenz Exp $
  * Copyright (c) 2008, Gaudenz Alder
  */
 package com.mxgraph.swing;
@@ -667,7 +667,7 @@ public class mxGraphOutline extends JComponent
 				g2.scale(scale, scale);
 
 				// Draws the scaled graph
-				graphComponent.paintGraph(g2, drawLabels);
+				graphComponent.getGraphControl().drawGraph(g2, drawLabels);
 			}
 			finally
 			{
