@@ -1,6 +1,6 @@
 /**
- * $Id: mxConstants.java,v 1.40 2010/01/12 10:09:44 gaudenz Exp $
- * Copyright (c) 2007, Gaudenz Alder
+ * $Id: mxConstants.java,v 1.62 2010-08-09 16:46:20 david Exp $
+ * Copyright (c) 2007-2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.util;
 
@@ -21,6 +21,22 @@ import javax.swing.border.LineBorder;
  */
 public class mxConstants
 {
+	/**
+	 * Defines the number of radians per degree.
+	 */
+	public static double RAD_PER_DEG = 0.0174532;
+
+	/**
+	 * Defines the number of degrees per radian.
+	 */
+	public static double DEG_PER_RAD = 57.2957795;
+
+	/**
+	 * Defines the minimum scale at which rounded polylines should be painted.
+	 * Default is 0.05.
+	 */
+	public static double MIN_SCALE_FOR_ROUNDED_LINES = 0.05;
+
 	/**
 	 * Defines the portion of the cell which is to be used as a connectable
 	 * region. Default is 0.3.
@@ -67,9 +83,10 @@ public class mxConstants
 	public static String DEFAULT_FONTFAMILIES = "Arial,Helvetica";
 
 	/**
-	 * Defines the default font family. Default is Arial.
+	 * Defines the default font family. Default is "Dialog". (To be replaced
+	 * with Font.DIALOG after EOL of Java 1.5.)
 	 */
-	public static String DEFAULT_FONTFAMILY = "Arial";
+	public static String DEFAULT_FONTFAMILY = "Dialog";
 
 	/**
 	 * Defines the default font size. Default is 11.
@@ -92,6 +109,12 @@ public class mxConstants
 	 */
 	public static int LABEL_INSET = 3;
 
+	/**
+	 * Multiplier to the width that is passed into the word wrapping calculation
+	 * See mxUtils.wordWrap for details
+	 */
+	public static double LABEL_SCALE_BUFFER = 0.9;
+
 	/**
 	 * Defines the default marker size. Default is 6.
 	 */
@@ -138,6 +161,17 @@ public class mxConstants
 	 */
 	public static Color DEFAULT_INVALID_COLOR = Color.RED;
 
+	/**
+	 * Specifies the default dash pattern, 3 pixels solid, 3 pixels clear.
+	 */
+	public static float[] DEFAULT_DASHED_PATTERN = new float[] {3.0f , 3.0f};
+
+	/**
+	 * Specifies the default distance at 1.0 scale that the label curve is 
+	 * created from its base curve
+	 */
+	public static double DEFAULT_LABEL_BUFFER = 12.0;
+
 	/**
 	 * Defines the rubberband border color. 
 	 */
@@ -149,10 +183,15 @@ public class mxConstants
 	public static Color RUBBERBAND_FILLCOLOR = new Color(51, 153, 255, 80);
 
 	/**
-	 * Defines the handle size. Default is 8.
+	 * Defines the handle size. Default is 7.
 	 */
 	public static int HANDLE_SIZE = 7;
 
+	/**
+	 * Defines the handle size. Default is 4.
+	 */
+	public static int LABEL_HANDLE_SIZE = 4;
+
 	/**
 	 * Defines the handle border color. Default is black.
 	 */
@@ -244,22 +283,22 @@ public class mxConstants
 
 	/**
 	 * Defines the length of the horizontal segment of an Entity Relation.
-	 * This can be overridden using mxConstants.STYLE_STARTSIZE style.
+	 * This can be overridden using mxConstants.STYLE_SEGMENT style.
 	 * Default is 30.
 	 */
 	public static int ENTITY_SEGMENT = 30;
-	
+
 	/**
 	 * Defines the rounding factor for rounded rectangles in percent between
 	 * 0 and 1. Values should be smaller than 0.5. Default is 0.15.
 	 */
 	public static double RECTANGLE_ROUNDING_FACTOR = 0.15;
-	
+
 	/**
 	 * Defines the size of the arcs for rounded edges. Default is 10.
 	 */
 	public static double LINE_ARCSIZE = 10;
-	
+
 	/**
 	 * Defines the spacing between the arrow shape and its terminals. Default
 	 * is 10.
@@ -291,6 +330,20 @@ public class mxConstants
 	 */
 	public static String STYLE_PERIMETER = "perimeter";
 
+	/**
+	 * Defines the ID of the cell that should be used for computing the
+	 * perimeter point of the source for an edge. This allows for graphically
+	 * connecting to a cell while keeping the actual terminal of the edge.
+	 */
+	public static String STYLE_SOURCE_PORT = "sourcePort";
+
+	/**
+	 * Defines the ID of the cell that should be used for computing the
+	 * perimeter point of the target for an edge. This allows for graphically
+	 * connecting to a cell while keeping the actual terminal of the edge.
+	 */
+	public static String STYLE_TARGET_PORT = "targetPort";
+
 	/**
 	 * Defines the key for the opacity style. The type of the value is 
 	 * <code>float</code> and the possible range is 0-100.
@@ -303,6 +356,75 @@ public class mxConstants
 	 */
 	public static String STYLE_TEXT_OPACITY = "textOpacity";
 
+	/**
+	 * Defines the key for the overflow style. Possible values are "visible",
+	 * "hidden" and "fill". The default value is "visible". This value
+	 * specifies how overlapping vertex labels are handles. A value of
+	 * "visible" will show the complete label. A value of "hidden" will clip
+	 * the label so that it does not overlap the vertex bounds. A value of
+	 * "fill" will use the vertex bounds for the label.
+	 * 
+	 * @see com.mxgraph.view.mxGraph#isLabelClipped(Object)
+	 */
+	public static String STYLE_OVERFLOW = "overflow";
+
+	/**
+	* Defines if the connection points on either end of the edge should be
+	* computed so that the edge is vertical or horizontal if possible and
+	* if the point is not at a fixed location. Default is false. This is
+	* used in mxGraph.isOrthogonal, which also returns true if the edgeStyle
+	* of the edge is an elbow or entity.
+	*/
+	public static String STYLE_ORTHOGONAL = "orthogonal";
+
+	/**
+	* Defines the key for the horizontal relative coordinate connection point
+	* of an edge with its source terminal.
+	*/
+	public static String STYLE_EXIT_X = "exitX";
+
+	/**
+	* Defines the key for the vertical relative coordinate connection point
+	* of an edge with its source terminal.
+	*/
+	public static String STYLE_EXIT_Y = "exitY";
+
+	/**
+	* Defines if the perimeter should be used to find the exact entry point
+	* along the perimeter of the source. Possible values are 0 (false) and
+	* 1 (true). Default is 1 (true).
+	*/
+	public static String STYLE_EXIT_PERIMETER = "exitPerimeter";
+
+	/**
+	* Defines the key for the horizontal relative coordinate connection point
+	* of an edge with its target terminal.
+	*/
+	public static String STYLE_ENTRY_X = "entryX";
+
+	/**
+	* Defines the key for the vertical relative coordinate connection point
+	* of an edge with its target terminal.
+	*/
+	public static String STYLE_ENTRY_Y = "entryY";
+
+	/**
+	* Defines if the perimeter should be used to find the exact entry point
+	* along the perimeter of the target. Possible values are 0 (false) and
+	* 1 (true). Default is 1 (true).
+	*/
+	public static String STYLE_ENTRY_PERIMETER = "entryPerimeter";
+
+	/**
+	 * Defines the key for the white-space style. Possible values are "nowrap"
+	 * and "wrap". The default value is "nowrap". This value specifies how
+	 * white-space inside a HTML vertex label should be handled. A value of
+	 * "nowrap" means the text will never wrap to the next line until a
+	 * linefeed is encountered. A value of "wrap" means text will wrap when
+	 * necessary.
+	 */
+	public static String STYLE_WHITE_SPACE = "whiteSpace";
+
 	/**
 	 * Defines the key for the rotation style. The type of the value is 
 	 * <code>double</code> and the possible range is 0-360.
@@ -313,7 +435,7 @@ public class mxConstants
 	 * Defines the key for the fillColor style. The value is a string
 	 * expression supported by mxUtils.parseColor.
 	 * 
-	 * @see com.mxgraph.util.mxUtils.parseColor(String)
+	 * @see com.mxgraph.util.mxUtils#parseColor(String)
 	 */
 	public static String STYLE_FILLCOLOR = "fillColor";
 
@@ -322,7 +444,7 @@ public class mxConstants
 	 * expression supported by mxUtils.parseColor. This is ignored if no fill
 	 * color is defined.
 	 * 
-	 * @see com.mxgraph.util.mxUtils.parseColor(String)
+	 * @see com.mxgraph.util.mxUtils#parseColor(String)
 	 */
 	public static String STYLE_GRADIENTCOLOR = "gradientColor";
 
@@ -343,7 +465,7 @@ public class mxConstants
 	 * Defines the key for the strokeColor style. The value is a string
 	 * expression supported by mxUtils.parseColor.
 	 * 
-	 * @see com.mxgraph.util.mxUtils.parseColor(String)
+	 * @see com.mxgraph.util.mxUtils#parseColor(String)
 	 */
 	public static String STYLE_STROKECOLOR = "strokeColor";
 
@@ -352,7 +474,7 @@ public class mxConstants
 	 * expression supported by mxUtils.parseColor. This style is only used
 	 * for SHAPE_SWIMLANE shapes.
 	 * 
-	 * @see com.mxgraph.util.mxUtils.parseColor(String)
+	 * @see com.mxgraph.util.mxUtils#parseColor(String)
 	 */
 	public static String STYLE_SEPARATORCOLOR = "separatorColor";
 
@@ -373,7 +495,7 @@ public class mxConstants
 	 * center of the text lines are aligned in the center of the label bounds.
 	 * Note this value doesn't affect the positioning of the overall label
 	 * bounds relative to the vertex, to move the label bounds horizontally, use
-	 * <code>STYLE_LABEL_POSITION</code>
+	 * <code>STYLE_LABEL_POSITION</code>.
 	 */
 	public static String STYLE_ALIGN = "align";
 
@@ -466,6 +588,18 @@ public class mxConstants
 	 */
 	public static String STYLE_IMAGE_HEIGHT = "imageHeight";
 
+	/**
+	 * Defines the key for the image background color. This style is only used
+	 * for image shapes. Possible values are all HTML color names or HEX codes.
+	 */
+	public static String STYLE_IMAGE_BACKGROUND = "imageBackground";
+
+	/**
+	 * Defines the key for the image border color. This style is only used for
+	 * image shapes. Possible values are all HTML color names or HEX codes.
+	 */
+	public static String STYLE_IMAGE_BORDER = "imageBorder";
+
 	/**
 	 * Defines the key for the noLabel style. If this is
 	 * true then no label is visible for a given cell.
@@ -486,7 +620,7 @@ public class mxConstants
 	 * Defines the key for the label background color. The value is a string
 	 * expression supported by mxUtils.parseColor.
 	 * 
-	 * @see com.mxgraph.util.mxUtils.parseColor(String)
+	 * @see com.mxgraph.util.mxUtils#parseColor(String)
 	 */
 	public static String STYLE_LABEL_BACKGROUNDCOLOR = "labelBackgroundColor";
 
@@ -494,7 +628,7 @@ public class mxConstants
 	 * Defines the key for the label border color. The value is a string
 	 * expression supported by mxUtils.parseColor.
 	 * 
-	 * @see com.mxgraph.util.mxUtils.parseColor(String)
+	 * @see com.mxgraph.util.mxUtils#parseColor(String)
 	 */
 	public static String STYLE_LABEL_BORDERCOLOR = "labelBorderColor";
 
@@ -516,7 +650,7 @@ public class mxConstants
 	 * Defines the key for the indicatorColor style. The value is a string
 	 * expression supported by mxUtils.parseColor.
 	 * 
-	 * @see com.mxgraph.util.mxUtils.parseColor(String)
+	 * @see com.mxgraph.util.mxUtils#parseColor(String)
 	 */
 	public static String STYLE_INDICATOR_COLOR = "indicatorColor";
 
@@ -525,7 +659,7 @@ public class mxConstants
 	 * string expression supported by mxUtils.parseColor. This style is only
 	 * supported in SHAPE_LABEL shapes.
 	 * 
-	 * @see com.mxgraph.util.mxUtils.parseColor(String)
+	 * @see com.mxgraph.util.mxUtils#parseColor(String)
 	 */
 	public static String STYLE_INDICATOR_GRADIENTCOLOR = "indicatorGradientColor";
 
@@ -551,6 +685,13 @@ public class mxConstants
 	 */
 	public static String STYLE_SHADOW = "shadow";
 
+	/**
+	 * Defines the key for the segment style. The type of this value is
+	 * <code>float</code> and the value represents the size of the horizontal
+	 * segment of the entity relation style. Default is ENTITY_SEGMENT.
+	 */
+	public static String STYLE_SEGMENT = "segment";
+
 	/**
 	 * Defines the key for the endArrow style.
 	 * Possible values are all constants in this
@@ -590,6 +731,13 @@ public class mxConstants
 	 */
 	public static String STYLE_DASHED = "dashed";
 
+	/**
+	 * Defines the key for the dashed pattern style. The type of this value
+	 * is <code>float[]</code> and the value specifies the dashed pattern 
+	 * to apply to edges drawn with this style.
+	 */
+	public static String STYLE_DASH_PATTERN = "dashPattern";
+
 	/**
 	 * Defines the key for the rounded style. The type of this value is
 	 * <code>boolean</code>. For edges this determines whether or not joins
@@ -694,7 +842,7 @@ public class mxConstants
 	 * <code>String</code> and of the expression supported by
 	 * mxUtils.parseColor.
 	 * 
-	 * @see com.mxgraph.util.mxUtils.parseColor(String)
+	 * @see com.mxgraph.util.mxUtils#parseColor(String)
 	 */
 	public static String STYLE_FONTCOLOR = "fontColor";
 
@@ -806,6 +954,11 @@ public class mxConstants
 	 */
 	public static final String SHAPE_ARROW = "arrow";
 
+	/**
+	 * SHAPE_ARROW
+	 */
+	public static final String SHAPE_CURVE = "curve";
+
 	/**
 	 * SHAPE_LABEL
 	 */
@@ -985,4 +1138,10 @@ public class mxConstants
 	 */
 	public static final String PERIMETER_TRIANGLE = "trianglePerimeter";
 
+	/**
+	 * Name of the hexagon perimeter. Can be used as a string value
+	 * for the STYLE_PERIMETER style.
+	 */
+	public static final String PERIMETER_HEXAGON  = "hexagonPerimeter";
+
 }
