@@ -1,6 +1,6 @@
 /**
- * $Id: mxGraphView.java,v 1.134 2010/01/13 10:43:46 gaudenz Exp $
- * Copyright (c) 2007, Gaudenz Alder
+ * $Id: mxGraphView.java,v 1.151 2010-08-02 13:44:58 david Exp $
+ * Copyright (c) 2007-2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.view;
 
@@ -11,6 +11,7 @@ import java.util.List;
 import java.util.Map;
 
 import com.mxgraph.model.mxGeometry;
+import com.mxgraph.model.mxGraphModel;
 import com.mxgraph.model.mxIGraphModel;
 import com.mxgraph.util.mxConstants;
 import com.mxgraph.util.mxEvent;
@@ -351,12 +352,12 @@ public class mxGraphView extends mxEventSource
 	}
 
 	/**
-	 * Removes all existing cell states and invokes revalidate.
+	 * Removes all existing cell states and invokes validate.
 	 */
 	public void reload()
 	{
 		states.clear();
-		revalidate();
+		validate();
 	}
 
 	/**
@@ -452,7 +453,14 @@ public class mxGraphView extends mxEventSource
 		if (cell != null)
 		{
 			validateBounds(null, cell);
-			setGraphBounds(validatePoints(null, cell));
+			mxRectangle bounds = validatePoints(null, cell);
+
+			if (bounds == null)
+			{
+				bounds = new mxRectangle();
+			}
+
+			setGraphBounds(bounds);
 		}
 	}
 
@@ -601,15 +609,9 @@ public class mxGraphView extends mxEventSource
 	 */
 	public mxRectangle validatePoints(mxCellState parentState, Object cell)
 	{
-		mxCellState state = getState(cell);
-
-		double minX = 0;
-		double minY = 0;
-		double maxX = 0;
-		double maxY = 0;
-
 		mxIGraphModel model = graph.getModel();
-		boolean isEdge = model.isEdge(cell);
+		mxCellState state = getState(cell);
+		mxRectangle bbox = null;
 
 		if (state != null)
 		{
@@ -617,31 +619,34 @@ public class mxGraphView extends mxEventSource
 			{
 				mxGeometry geo = graph.getCellGeometry(cell);
 
-				if (isEdge)
+				if (geo != null && model.isEdge(cell))
 				{
-					Object source = getVisibleTerminal(cell, true);
+					// Updates the points on the source terminal if its an edge
+					mxCellState source = getState(getVisibleTerminal(cell, true));
 
-					if (source != null && !model.isAncestor(source, cell))
+					if (source != null && model.isEdge(source.getCell())
+							&& !model.isAncestor(source, cell))
 					{
-						Object p = model.getParent(source);
-						mxCellState pstate = getState(p);
-
-						validatePoints(pstate, source);
+						mxCellState tmp = getState(model.getParent(source
+								.getCell()));
+						validatePoints(tmp, source);
 					}
 
-					Object target = getVisibleTerminal(cell, false);
+					// Updates the points on the source terminal if its an edge
+					mxCellState target = getState(getVisibleTerminal(cell,
+							false));
 
-					if (target != null && !model.isAncestor(target, cell))
+					if (target != null && model.isEdge(target.getCell())
+							&& !model.isAncestor(target, cell))
 					{
-						Object p = model.getParent(target);
-						mxCellState pstate = getState(p);
-
-						validatePoints(pstate, target);
+						mxCellState tmp = getState(model.getParent(target
+								.getCell()));
+						validatePoints(tmp, target);
 					}
 
-					setTerminalPoints(state);
+					updateFixedTerminalPoints(state, source, target);
 					updatePoints(state, geo.getPoints(), source, target);
-					updateTerminalPoints(state, source, target);
+					updateFloatingTerminalPoints(state, source, target);
 					updateEdgeBounds(state);
 					state.setAbsoluteOffset(getPoint(state, geo));
 				}
@@ -666,15 +671,10 @@ public class mxGraphView extends mxEventSource
 				state.setInvalid(false);
 			}
 
-			if (isEdge || model.isVertex(cell))
+			if (model.isEdge(cell) || model.isVertex(cell))
 			{
 				updateLabelBounds(state);
-				mxRectangle bb = updateBoundingBox(state);
-
-				minX = bb.getX();
-				minY = bb.getY();
-				maxX = bb.getX() + bb.getWidth();
-				maxY = bb.getY() + bb.getHeight();
+				bbox = new mxRectangle(updateBoundingBox(state));
 			}
 		}
 
@@ -688,15 +688,21 @@ public class mxGraphView extends mxEventSource
 				Object child = model.getChildAt(cell, i);
 				mxRectangle bounds = validatePoints(state, child);
 
-				// TODO: Fix initial 0 for minX, minY (should be null)
-				minX = Math.min(minX, bounds.getX());
-				minY = Math.min(minY, bounds.getY());
-				maxX = Math.max(maxX, bounds.getX() + bounds.getWidth());
-				maxY = Math.max(maxY, bounds.getY() + bounds.getHeight());
+				if (bounds != null)
+				{
+					if (bbox == null)
+					{
+						bbox = bounds;
+					}
+					else
+					{
+						bbox.add(bounds);
+					}
+				}
 			}
 		}
 
-		return new mxRectangle(minX, minY, maxX - minX, maxY - minY);
+		return bbox;
 	}
 
 	/**
@@ -706,9 +712,9 @@ public class mxGraphView extends mxEventSource
 	 * are updated in validatePoints, and validates the bounds of all
 	 * descendants of the child using validateBounds.
 	 */
-	protected void childMoved(mxCellState parentState, mxCellState childState)
+	protected void childMoved(mxCellState parent, mxCellState child)
 	{
-		Object cell = childState.getCell();
+		Object cell = child.getCell();
 
 		// Children of relative edge children need to validate
 		// their bounds after their parent state was updated
@@ -719,7 +725,7 @@ public class mxGraphView extends mxEventSource
 
 			for (int i = 0; i < childCount; i++)
 			{
-				validateBounds(childState, model.getChildAt(cell, i));
+				validateBounds(child, model.getChildAt(cell, i));
 			}
 		}
 	}
@@ -731,12 +737,21 @@ public class mxGraphView extends mxEventSource
 	{
 		Object cell = state.getCell();
 		Map<String, Object> style = state.getStyle();
-		String label = graph.getLabel(cell);
-		mxRectangle vertexBounds = (!graph.getModel().isEdge(cell)) ? state
-				: null;
-		state.setLabelBounds(mxUtils.getLabelPaintBounds(label, style, graph
-				.isHtmlLabel(cell), state.getAbsoluteOffset(), vertexBounds,
-				scale));
+
+		if (mxUtils.getString(style, mxConstants.STYLE_OVERFLOW, "").equals(
+				"fill"))
+		{
+			state.setLabelBounds(new mxRectangle(state));
+		}
+		else
+		{
+			String label = graph.getLabel(cell);
+			mxRectangle vertexBounds = (!graph.getModel().isEdge(cell)) ? state
+					: null;
+			state.setLabelBounds(mxUtils.getLabelPaintBounds(label, style,
+					graph.isHtmlLabel(cell), state.getAbsoluteOffset(),
+					vertexBounds, scale));
+		}
 	}
 
 	/**
@@ -754,9 +769,10 @@ public class mxGraphView extends mxEventSource
 		// Adds extra pixels for the marker and stroke assuming
 		// that the border stroke is centered around the bounds
 		// and the first pixel is drawn inside the bounds
-		double strokeWidth = Math.max(1, Math.round(mxUtils.getInt(style,
-				mxConstants.STYLE_STROKEWIDTH, 1)
-				* scale));
+		double strokeWidth = Math.max(
+				1,
+				Math.round(mxUtils.getInt(style, mxConstants.STYLE_STROKEWIDTH,
+						1) * scale));
 		strokeWidth -= Math.max(1, strokeWidth / 2);
 
 		if (graph.getModel().isEdge(state.getCell()))
@@ -798,8 +814,7 @@ public class mxGraphView extends mxEventSource
 			if (mxUtils.getString(style, mxConstants.STYLE_IMAGE) != null)
 			{
 				double w = mxUtils.getInt(style, mxConstants.STYLE_IMAGE_WIDTH,
-						mxConstants.DEFAULT_IMAGESIZE)
-						* scale;
+						mxConstants.DEFAULT_IMAGESIZE) * scale;
 				double h = mxUtils.getInt(style,
 						mxConstants.STYLE_IMAGE_HEIGHT,
 						mxConstants.DEFAULT_IMAGESIZE)
@@ -846,12 +861,9 @@ public class mxGraphView extends mxEventSource
 		double rotation = mxUtils.getDouble(style, mxConstants.STYLE_ROTATION);
 		mxRectangle bbox = mxUtils.getBoundingBox(rect, rotation);
 
-		// Add the rotated bounding box to the non-rotated so
+		// Adda the rotated bounding box to the non-rotated so
 		// that all handles are also covered
-		if (bbox != null)
-		{
-			rect.add(bbox);
-		}
+		rect.add(bbox);
 
 		// Unifies the cell bounds and the label bounds
 		if (!graph.isLabelClipped(state.getCell()))
@@ -865,82 +877,92 @@ public class mxGraphView extends mxEventSource
 	}
 
 	/**
-	 * Sets the initial absolute terminal points in the given state.
+	 * Sets the initial absolute terminal points in the given state before the edge
+	 * style is computed.
 	 * 
-	 * @param state Cell state whose initial terminal points should be
+	 * @param edge Cell state whose initial terminal points should be updated.
+	 * @param source Cell state which represents the source terminal.
+	 * @param target Cell state which represents the target terminal.
+	 */
+	public void updateFixedTerminalPoints(mxCellState edge, mxCellState source,
+			mxCellState target)
+	{
+		updateFixedTerminalPoint(edge, source, true,
+				graph.getConnectionConstraint(edge, source, true));
+		updateFixedTerminalPoint(edge, target, false,
+				graph.getConnectionConstraint(edge, target, false));
+	}
+
+	/**
+	 * Sets the fixed source or target terminal point on the given edge.
+	 * 
+	 * @param edge Cell state whose initial terminal points should be
 	 * updated.
 	 */
-	public void setTerminalPoints(mxCellState state)
+	public void updateFixedTerminalPoint(mxCellState edge,
+			mxCellState terminal, boolean source,
+			mxConnectionConstraint constraint)
 	{
-		Object edge = state.getCell();
+		mxPoint pt = null;
 
-		mxGeometry geo = graph.getCellGeometry(edge);
-		mxPoint orig = state.getOrigin();
-
-		mxPoint pt = geo.getTerminalPoint(true);
-
-		if (pt != null)
+		if (constraint != null)
 		{
-			pt = new mxPoint(scale
-					* (translate.getX() + pt.getX() + orig.getX()), scale
-					* (translate.getY() + pt.getY() + orig.getY()));
-			state.setAbsoluteTerminalPoint(pt, true);
-		}
-		else
-		{
-			state.setAbsoluteTerminalPoint(null, true);
+			pt = graph.getConnectionPoint(terminal, constraint);
 		}
 
-		pt = geo.getTerminalPoint(false);
+		if (pt == null && terminal == null)
+		{
+			mxPoint orig = edge.getOrigin();
+			mxGeometry geo = graph.getCellGeometry(edge.cell);
+			pt = geo.getTerminalPoint(source);
 
-		if (pt != null)
-		{
-			pt = new mxPoint(scale
-					* (translate.getX() + pt.getX() + orig.getX()), scale
-					* (translate.getY() + pt.getY() + orig.getY()));
-			state.setAbsoluteTerminalPoint(pt, false);
-		}
-		else
-		{
-			state.setAbsoluteTerminalPoint(null, false);
+			if (pt != null)
+			{
+				pt = new mxPoint(scale
+						* (translate.getX() + pt.getX() + orig.getX()), scale
+						* (translate.getY() + pt.getY() + orig.getY()));
+			}
 		}
+
+		edge.setAbsoluteTerminalPoint(pt, source);
 	}
 
 	/**
 	 * Updates the absolute points in the given state using the specified array
 	 * of points as the relative points.
 	 * 
-	 * @param state Cell state whose absolute points should be updated.
+	 * @param edge Cell state whose absolute points should be updated.
 	 * @param points Array of points that constitute the relative points.
-	 * @param source Cell that represents the visual source.
-	 * @param target Cell that represents the visual target.
+	 * @param source Cell state that represents the source terminal.
+	 * @param target Cell state that represents the target terminal.
 	 */
-	public void updatePoints(mxCellState state, List<mxPoint> points,
-			Object source, Object target)
+	public void updatePoints(mxCellState edge, List<mxPoint> points,
+			mxCellState source, mxCellState target)
 	{
-		if (state != null)
+		if (edge != null)
 		{
 			List<mxPoint> pts = new ArrayList<mxPoint>();
-			pts.add(state.getAbsolutePoint(0));
-			mxEdgeStyleFunction edgeStyle = getEdgeStyle(state, points, source,
+			pts.add(edge.getAbsolutePoint(0));
+			mxEdgeStyleFunction edgeStyle = getEdgeStyle(edge, points, source,
 					target);
 
 			if (edgeStyle != null)
 			{
-				mxCellState src = getState(source);
-				mxCellState trg = getState(target);
-				edgeStyle.apply(state, src, trg, points, pts);
+				mxCellState src = getTerminalPort(edge, source, true);
+				mxCellState trg = getTerminalPort(edge, target, false);
+
+				edgeStyle.apply(edge, src, trg, points, pts);
 			}
 			else if (points != null)
 			{
 				for (int i = 0; i < points.size(); i++)
 				{
-					pts.add(transformControlPoint(state, points.get(i)));
+					pts.add(transformControlPoint(edge, points.get(i)));
 				}
 			}
 
-			pts.add(state.getAbsolutePoint(state.getAbsolutePointCount() - 1));
-			state.setAbsolutePoints(pts);
+			pts.add(edge.getAbsolutePoint(edge.getAbsolutePointCount() - 1));
+			edge.setAbsolutePoints(pts);
 		}
 	}
 
@@ -951,32 +973,33 @@ public class mxGraphView extends mxEventSource
 	{
 		mxPoint origin = state.getOrigin();
 
-		return new mxPoint((pt.getX() + translate.getX() + origin.getX())
-				* scale, (pt.getY() + translate.getY() + origin.getY()) * scale);
+		return new mxPoint(scale
+				* (pt.getX() + translate.getX() + origin.getX()), scale
+				* (pt.getY() + translate.getY() + origin.getY()));
 	}
 
 	/**
 	 * Returns the edge style function to be used to compute the absolute
 	 * points for the given state, control points and terminals.
 	 */
-	public mxEdgeStyleFunction getEdgeStyle(mxCellState edgeState,
+	public mxEdgeStyleFunction getEdgeStyle(mxCellState edge,
 			List<mxPoint> points, Object source, Object target)
 	{
 		Object edgeStyle = null;
 
 		if (source != null && source == target)
 		{
-			edgeStyle = edgeState.getStyle().get(mxConstants.STYLE_LOOP);
+			edgeStyle = edge.getStyle().get(mxConstants.STYLE_LOOP);
 
 			if (edgeStyle == null)
 			{
 				edgeStyle = graph.getDefaultLoopStyle();
 			}
 		}
-		else if (!mxUtils.isTrue(edgeState.getStyle(),
+		else if (!mxUtils.isTrue(edge.getStyle(),
 				mxConstants.STYLE_NOEDGESTYLE, false))
 		{
-			edgeStyle = edgeState.getStyle().get(mxConstants.STYLE_EDGE);
+			edgeStyle = edge.getStyle().get(mxConstants.STYLE_EDGE);
 		}
 
 		// Converts string values to objects
@@ -1002,82 +1025,124 @@ public class mxGraphView extends mxEventSource
 	}
 
 	/**
-	 * Updates the terminal points in the given state.
+	 * Updates the terminal points in the given state after the edge style was
+	 * computed for the edge.
 	 * 
 	 * @param state Cell state whose terminal points should be updated.
-	 * @param source Cell that represents the visual source.
-	 * @param target Cell that represents the visual target.
+	 * @param source Cell state that represents the source terminal.
+	 * @param target Cell state that represents the target terminal.
 	 */
-	public void updateTerminalPoints(mxCellState state, Object source,
-			Object target)
+	public void updateFloatingTerminalPoints(mxCellState state,
+			mxCellState source, mxCellState target)
 	{
-		if (target != null)
+		mxPoint p0 = state.getAbsolutePoint(0);
+		mxPoint pe = state.getAbsolutePoint(state.getAbsolutePointCount() - 1);
+
+		if (pe == null && target != null)
 		{
-			updateTerminalPoint(state, target, source, false);
+			updateFloatingTerminalPoint(state, target, source, false);
 		}
 
-		if (source != null)
+		if (p0 == null && source != null)
 		{
-			updateTerminalPoint(state, source, target, true);
+			updateFloatingTerminalPoint(state, source, target, true);
 		}
 	}
 
 	/**
 	 * Updates the absolute terminal point in the given state for the given
-	 * start and end state, where start is the source if isSource is true.
+	 * start and end state, where start is the source if source is true.
 	 * 
-	 * @param state Cell state whose terminal point should be updated.
-	 * @param start Cell state for the source or target terminal.
-	 * @param end Cell state for the opposite terminal.
-	 * @param isSource Boolean indicating if start is the state of the source
-	 * terminal.
+	 * @param edge Cell state whose terminal point should be updated.
+	 * @param start Cell state for the terminal on "this" side of the edge.
+	 * @param end Cell state for the terminal on the other side of the edge.
+	 * @param source Boolean indicating if start is the source terminal state.
 	 */
-	public void updateTerminalPoint(mxCellState state, Object start,
-			Object end, boolean isSource)
+	public void updateFloatingTerminalPoint(mxCellState edge,
+			mxCellState start, mxCellState end, boolean source)
 	{
-		int index = (isSource) ? 0 : state.getAbsolutePointCount() - 1;
-		state.setAbsolutePoint(index, getPerimeterPoint(state, start, end,
-				isSource));
+		start = getTerminalPort(edge, start, source);
+		mxPoint next = getNextPoint(edge, end, source);
+		double border = mxUtils.getDouble(edge.getStyle(),
+				mxConstants.STYLE_PERIMETER_SPACING);
+		border += mxUtils.getDouble(edge.getStyle(),
+				(source) ? mxConstants.STYLE_SOURCE_PERIMETER_SPACING
+						: mxConstants.STYLE_TARGET_PERIMETER_SPACING);
+		mxPoint pt = getPerimeterPoint(start, next, graph.isOrthogonal(edge),
+				border);
+		edge.setAbsoluteTerminalPoint(pt, source);
 	}
 
 	/**
-	 * Returns a point that defines the location of the connection point
-	 * between the edge represented by the given state and the source or target
-	 * end of the edge, depending on isSource.
-	 * 
-	 * @param state Cell state of the connecting edge.
-	 * @param start Cell state for the source or target terminal.
-	 * @param end Cell state for the opposite terminal.
-	 * @param isSource Boolean indicating if start is the state of the source
-	 * terminal.
-	 * @return Returns the connection point between the edge and the terminal.
+	 * Returns a cell state that represents the source or target terminal or
+	 * port for the given edge.
 	 */
-	public mxPoint getPerimeterPoint(mxCellState state, Object start,
-			Object end, boolean isSource)
+	public mxCellState getTerminalPort(mxCellState state, mxCellState terminal,
+			boolean source)
+	{
+		String key = (source) ? mxConstants.STYLE_SOURCE_PORT
+				: mxConstants.STYLE_TARGET_PORT;
+		String id = mxUtils.getString(state.style, key);
+
+		if (id != null && graph.getModel() instanceof mxGraphModel)
+		{
+			mxCellState tmp = getState(((mxGraphModel) graph.getModel())
+					.getCell(id));
+
+			// Only uses ports where a cell state exists
+			if (tmp != null)
+			{
+				terminal = tmp;
+			}
+		}
+
+		return terminal;
+	}
+
+	/**
+	 * Returns a point that defines the location of the intersection point between
+	 * the perimeter and the line between the center of the shape and the given point.
+	 */
+	public mxPoint getPerimeterPoint(mxCellState terminal, mxPoint next,
+			boolean orthogonal)
+	{
+		return getPerimeterPoint(terminal, next, orthogonal, 0);
+	}
+
+	/**
+	 * Returns a point that defines the location of the intersection point between
+	 * the perimeter and the line between the center of the shape and the given point.
+	 * 
+	 * @param terminal Cell state for the source or target terminal.
+	 * @param next Point that lies outside of the given terminal.
+	 * @param orthogonal Boolean that specifies if the orthogonal projection onto
+	 * the perimeter should be returned. If this is false then the intersection
+	 * of the perimeter and the line between the next and the center point is
+	 * returned.
+	 * @param border Optional border between the perimeter and the shape.
+	 */
+	public mxPoint getPerimeterPoint(mxCellState terminal, mxPoint next,
+			boolean orthogonal, double border)
 	{
 		mxPoint point = null;
-		mxCellState terminalState = getState(start);
 
-		if (terminalState != null)
+		if (terminal != null)
 		{
-			mxPerimeterFunction perimeter = getPerimeterFunction(terminalState);
-			mxPoint next = getNextPoint(state, end, isSource);
+			mxPerimeterFunction perimeter = getPerimeterFunction(terminal);
 
 			if (perimeter != null && next != null)
 			{
-				mxRectangle bounds = getPerimeterBounds(terminalState, state,
-						isSource);
+				mxRectangle bounds = getPerimeterBounds(terminal, border);
 
 				if (bounds.getWidth() > 0 || bounds.getHeight() > 0)
 				{
-					point = perimeter.apply(bounds, state, terminalState,
-							isSource, next);
+					point = perimeter.apply(bounds, terminal, next, orthogonal);
 				}
 			}
 
 			if (point == null)
 			{
-				point = getPoint(terminalState, null);
+				point = getPoint(terminal);
 			}
 		}
 
@@ -1091,8 +1156,8 @@ public class mxGraphView extends mxEventSource
 	 */
 	public double getRoutingCenterX(mxCellState state)
 	{
-		float f = (state.getStyle() != null) ? mxUtils.getFloat(state
-				.getStyle(), mxConstants.STYLE_ROUTING_CENTER_X) : 0;
+		float f = (state.getStyle() != null) ? mxUtils.getFloat(
+				state.getStyle(), mxConstants.STYLE_ROUTING_CENTER_X) : 0;
 
 		return state.getCenterX() + f * state.getWidth();
 	}
@@ -1104,8 +1169,8 @@ public class mxGraphView extends mxEventSource
 	 */
 	public double getRoutingCenterY(mxCellState state)
 	{
-		float f = (state.getStyle() != null) ? mxUtils.getFloat(state
-				.getStyle(), mxConstants.STYLE_ROUTING_CENTER_Y) : 0;
+		float f = (state.getStyle() != null) ? mxUtils.getFloat(
+				state.getStyle(), mxConstants.STYLE_ROUTING_CENTER_Y) : 0;
 
 		return state.getCenterY() + f * state.getHeight();
 	}
@@ -1113,20 +1178,8 @@ public class mxGraphView extends mxEventSource
 	/**
 	 * Returns the perimeter bounds for the given terminal, edge pair.
 	 */
-	public mxRectangle getPerimeterBounds(mxCellState terminal,
-			mxCellState edge, boolean isSource)
+	public mxRectangle getPerimeterBounds(mxCellState terminal, double border)
 	{
-		double border = 0;
-
-		if (edge != null)
-		{
-			border = mxUtils.getDouble(edge.getStyle(),
-					mxConstants.STYLE_PERIMETER_SPACING);
-			border += mxUtils.getDouble(edge.getStyle(),
-					(isSource) ? mxConstants.STYLE_SOURCE_PERIMETER_SPACING
-							: mxConstants.STYLE_TARGET_PERIMETER_SPACING);
-		}
-
 		if (terminal != null)
 		{
 			border += mxUtils.getDouble(terminal.getStyle(),
@@ -1169,35 +1222,29 @@ public class mxGraphView extends mxEventSource
 	 * Returns the nearest point in the list of absolute points or the center
 	 * of the opposite terminal.
 	 * 
-	 * @param state Cell state that represents the edge.
-	 * @param opposite Cell that represents the opposite terminal.
-	 * @param isSource Boolean indicating if the next point for the source or target
+	 * @param edge Cell state that represents the edge.
+	 * @param opposite Cell state that represents the opposite terminal.
+	 * @param source Boolean indicating if the next point for the source or target
 	 * should be returned.
 	 * @return Returns the nearest point of the opposite side.
 	 */
-	public mxPoint getNextPoint(mxCellState state, Object opposite,
-			boolean isSource)
+	public mxPoint getNextPoint(mxCellState edge, mxCellState opposite,
+			boolean source)
 	{
+		List<mxPoint> pts = edge.getAbsolutePoints();
 		mxPoint point = null;
-		List<mxPoint> pts = state.getAbsolutePoints();
 
-		if (pts != null && (isSource || pts.size() > 2 || opposite == null))
+		if (pts != null && (source || pts.size() > 2 || opposite == null))
 		{
 			int count = pts.size();
-			int index = (isSource) ? Math.min(1, count - 1) : Math.max(0,
+			int index = (source) ? Math.min(1, count - 1) : Math.max(0,
 					count - 2);
 			point = pts.get(index);
 		}
 
 		if (point == null && opposite != null)
 		{
-			mxCellState oppositeState = getState(opposite);
-
-			if (oppositeState != null)
-			{
-				point = new mxPoint(oppositeState.getCenterX(), oppositeState
-						.getCenterY());
-			}
+			point = new mxPoint(opposite.getCenterX(), opposite.getCenterY());
 		}
 
 		return point;
@@ -1208,14 +1255,14 @@ public class mxGraphView extends mxEventSource
 	 * to be connected to this terminal on the display.
 	 * 
 	 * @param edge Cell whose visible terminal should be returned.
-	 * @param isSource Boolean that specifies if the source or target terminal
+	 * @param source Boolean that specifies if the source or target terminal
 	 * should be returned.
 	 * @return Returns the visible source or target terminal.
 	 */
-	public Object getVisibleTerminal(Object edge, boolean isSource)
+	public Object getVisibleTerminal(Object edge, boolean source)
 	{
 		mxIGraphModel model = graph.getModel();
-		Object result = model.getTerminal(edge, isSource);
+		Object result = model.getTerminal(edge, source);
 		Object best = result;
 
 		while (result != null && result != currentRoot)
@@ -1277,47 +1324,40 @@ public class mxGraphView extends mxEventSource
 				double[] segments = new double[points.size() - 1];
 				mxPoint pt = p0;
 
-				if (pt != null)
-				{
-					double minX = pt.getX();
-					double minY = pt.getY();
-					double maxX = minX;
-					double maxY = minY;
+				double minX = pt.getX();
+				double minY = pt.getY();
+				double maxX = minX;
+				double maxY = minY;
 
-					for (int i = 1; i < points.size(); i++)
+				for (int i = 1; i < points.size(); i++)
+				{
+					mxPoint tmp = points.get(i);
+
+					if (tmp != null)
 					{
-						mxPoint tmp = points.get(i);
+						double dx = pt.getX() - tmp.getX();
+						double dy = pt.getY() - tmp.getY();
 
-						if (tmp != null)
-						{
-							double dx = pt.getX() - tmp.getX();
-							double dy = pt.getY() - tmp.getY();
+						double segment = Math.sqrt(dx * dx + dy * dy);
+						segments[i - 1] = segment;
+						length += segment;
+						pt = tmp;
 
-							double segment = Math.sqrt(dx * dx + dy * dy);
-							segments[i - 1] = segment;
-							length += segment;
-							pt = tmp;
-
-							minX = Math.min(pt.getX(), minX);
-							minY = Math.min(pt.getY(), minY);
-							maxX = Math.max(pt.getX(), maxX);
-							maxY = Math.max(pt.getY(), maxY);
-						}
+						minX = Math.min(pt.getX(), minX);
+						minY = Math.min(pt.getY(), minY);
+						maxX = Math.max(pt.getX(), maxX);
+						maxY = Math.max(pt.getY(), maxY);
 					}
-
-					state.setLength(length);
-					state.setSegments(segments);
-					double markerSize = 1; // TODO: include marker size
-
-					state.setX(minX);
-					state.setY(minY);
-					state.setWidth(Math.max(markerSize, maxX - minX));
-					state.setHeight(Math.max(markerSize, maxY - minY));
-				}
-				else
-				{
-					state.setLength(0);
 				}
+
+				state.setLength(length);
+				state.setSegments(segments);
+				double markerSize = 1; // TODO: include marker size
+
+				state.setX(minX);
+				state.setY(minY);
+				state.setWidth(Math.max(markerSize, maxX - minX));
+				state.setHeight(Math.max(markerSize, maxY - minY));
 			}
 		}
 	}
@@ -1496,10 +1536,10 @@ public class mxGraphView extends mxEventSource
 					projlen = seg;
 				}
 
-				double yDistance = Line2D.ptLineDist(p0.getX(), p0.getY(), pe
-						.getX(), pe.getY(), x, y);
-				int direction = Line2D.relativeCCW(p0.getX(), p0.getY(), pe
-						.getX(), pe.getY(), x, y);
+				double yDistance = Line2D.ptLineDist(p0.getX(), p0.getY(),
+						pe.getX(), pe.getY(), x, y);
+				int direction = Line2D.relativeCCW(p0.getX(), p0.getY(),
+						pe.getX(), pe.getY(), x, y);
 
 				if (direction == -1)
 				{
@@ -1508,9 +1548,8 @@ public class mxGraphView extends mxEventSource
 
 				// Constructs the relative point for the label
 				return new mxPoint(
-						Math
-								.round(((totalLength / 2 - length - projlen) / totalLength)
-										* -2), Math.round(yDistance / scale));
+						Math.round(((totalLength / 2 - length - projlen) / totalLength)
+								* -2), Math.round(yDistance / scale));
 			}
 		}
 
@@ -1536,7 +1575,8 @@ public class mxGraphView extends mxEventSource
 			}
 		}
 
-		return (mxCellState[]) result.toArray();
+		mxCellState[] resultArray = new mxCellState[result.size()];
+		return result.toArray(resultArray);
 	}
 
 	/**
@@ -1566,7 +1606,7 @@ public class mxGraphView extends mxEventSource
 
 		if (cell != null)
 		{
-			state = (mxCellState) states.get(cell);
+			state = states.get(cell);
 
 			if (state == null && create && graph.isCellVisible(cell))
 			{
@@ -1695,8 +1735,8 @@ public class mxGraphView extends mxEventSource
 
 			if (translate != null)
 			{
-				view.translate = new mxPoint(-translate.getX(), translate
-						.getY());
+				view.translate = new mxPoint(-translate.getX(),
+						translate.getY());
 			}
 
 			// Removes all existing cell states and revalidates
